'''
Author: hexu
Date: 2021-10-14 14:54:05
LastEditTime: 2023-05-19 16:10:09
LastEditors: Hexu
Description: 从智能服务获取模型数据集
FilePath: /iw-algo-fx/intelliw/datasets/datasource_intelliv.py
'''
from concurrent.futures import ThreadPoolExecutor
import os
import time

from intelliw.datasets.datasource_base import AbstractDataSource, DataSourceReaderException, \
    DatasetSelector, DataSourceType

from intelliw.utils import iuap_request
from intelliw.utils.logger import _get_framework_logger
from intelliw.config import config

logger = _get_framework_logger()


class DataSourceIntelliv(AbstractDataSource):
    """
    智能分析数据源
    """

    def __init__(self, input_address, get_row_address, input_model_id):
        """
        智能分析数据源
        :param input_address:   获取数据 url
        :param get_row_address: 获取数据总条数 url
        :param input_model_id:  读取的数据模型
        """
        self.input_address = input_address
        self.input_model_id = input_model_id
        self.get_row_address = f"{get_row_address}/{input_model_id}/null"

        self.__total = None

    def total(self):
        if self.__total is not None:
            return self.__total
        response = iuap_request.get(
            self.get_row_address, auth_type=iuap_request.AuthType.YHT, timeout=config.DATA_SOURCE_READ_TIMEOUT)

        if 200 != response.status or response.json is None:
            msg = "获取行数失败，url: {}, response: {}".format(
                self.get_row_address, response)
            raise DataSourceReaderException(msg)

        row_data = response.json
        self.__total = row_data['data']
        if not isinstance(self.__total, int):
            msg = f"获取行数返回结果错误, response: {row_data}, request_url: {self.get_row_address}"
            raise DataSourceReaderException(msg)
        return self.__total

    def reader(self, page_size=100000, offset=0, limit=0, transform_function=None, dataset_type='train_set'):
        return self.__Reader(self.input_address, self.input_model_id, self.total(), page_size, offset, limit,
                             transform_function)

    class __Reader:
        def __init__(self, input_address, input_model_id, total, page_size=10000, offset=0, limit=0,
                     transform_function=None):
            """
            eg. 91 elements, page_size = 20, 5 pages as below:
            [0,19][20,39][40,59][60,79][80,90]
            offset 15, limit 30:
            [15,19][20,39][40,44]
            offset 10 limit 5:
            [10,14]
            """
            self.input_address = input_address
            self.input_model_id = input_model_id
            self.total_rows = total
            self.total_read = 0
            self.page_size = max(100, page_size)
            self.page_num = 1
            self.meta = []
            self.after_transform = 0
            self.transform_function = transform_function
            self.worker = min(3, max(1, os.cpu_count()))
            logger.info('数据获取中: 启用的获取线程 %s 条', self.worker)

        def get_data_bar(self):
            """数据拉取进度条"""
            try:
                proportion = round(
                    (self.total_read / self.total_rows) * 100, 2)
                logger.info(
                    "数据获取中: 共%s条数据, 已获取%s条, 进度%s%%", self.total_rows, self.total_read, proportion)
            except:
                pass

        @property
        def iterable(self):
            return True

        def __iter__(self):
            return self

        def __next__(self):
            if self.total_read >= self.total_rows:
                logger.info('数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                raise StopIteration

            self.get_data_bar()

            try:
                with ThreadPoolExecutor(max_workers=self.worker) as executor:
                    futures, data_result = [], []
                    for _ in range(self.worker * 2):
                        futures.append(executor.submit(
                            self._read_page, self.page_num, self.page_size
                        ))
                        self.page_num += 1

                    for task in futures:
                        data_result.extend(task.result())
                    self.total_read += len(data_result)
                    if len(data_result) == 0:
                        logger.info(
                            '数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                        raise StopIteration

                    return_data = {"result": data_result, "meta": self.meta}
                    if self.transform_function is not None:
                        return_data = self.transform_function(return_data)
                    return return_data
            except StopIteration:
                raise StopIteration
            except Exception as e:
                logger.exception("智能分析数据源读取失败, input_address: [{}], model_id: [{}]".
                                 format(self.input_address, self.input_model_id))
                raise DataSourceReaderException(f'智能分析数据源读取失败:{e}')

        def _get_meta(self, meta):
            if not self.meta:
                self.meta = meta
            return self.meta

        def _data_process(self, data):
            assert data is not None, "智能分析数据接口未返回任何有效数据"
            if len(data) > 0:
                self._get_meta(data['meta'])
            return data['result']

        def _read_page(self, page_index, page_size):
            """
            调用智能分析接口，分页读取数据
            :param page_index: 页码，从 1 开始
            :param page_size:  每页大小
            :return:
            """
            request_data = {
                "modelId": self.input_model_id,
                "pager": {
                    "pageIndex": page_index,
                    "pageSize": page_size,
                    "pageCount": 0,
                    "recordCount": 0
                },
                "signDTO": {"ak": config.ACCESS_KEY,
                            "sign": config.ACCESS_SECRET,
                            "ts": int(time.time() * 1000)}
            }
            response = iuap_request.post_json(
                url=self.input_address, json=request_data, timeout=config.DATA_SOURCE_READ_TIMEOUT,
                auth_type=iuap_request.AuthType.YHT)
            response.raise_for_status()
            return self._data_process(response.json)


DatasetSelector.register_func(DataSourceType.INTELLIV, DataSourceIntelliv, {
    "input_address": "INPUT_ADDR",
    "get_row_address": "INPUT_GETROW_ADDR",
    "input_model_id": "INPUT_MODEL_ID"})



import json
import time

from intelliw.utils.kafka import get_client, handle_message
import os

os.environ['kafka.cluster.hosts'] = '172.20.32.80:9092,172.20.32.82:9092,172.20.32.83:9092'


def test_get_client_with_args(group_id):
    """
    测试通过直接传参获取 Kafka 配置
    """
    return get_client(
        brokers='localhost:9092',
        group_id=group_id,
        username='custom_user',
        password='custom_pass',
        producer_config={'acks': 'all'},
        consumer_config={'auto.offset.reset': 'earliest',
                         'enable.auto.commit': True,
                         'max.poll.interval.ms': 8000000,
                         'heartbeat.interval.ms': 3000,
                         'session.timeout.ms': 30000}
    )


def test_get_client_with_env(group_id):
    """
    测试通过直接传参获取 Kafka 配置
    """
    return get_client(
        group_id="test.tenders.2",
        consumer_config={'auto.offset.reset': 'earliest',
                         'enable.auto.commit': True,
                         'max.poll.interval.ms': 8000000,
                         'heartbeat.interval.ms': 3000,
                         'session.timeout.ms': 30000
                         }
    )


class MyConsumer:
    def __init__(self):
        # 初始化代码
        pass

    def return_func(self, k, v):
        return True
        # data = json.loads(v)
        # id = data['data'][0]['id']
        # if id == "2118076642662809607":
        #     return True
        # return False

    def handle_message(self, k, v):
        # 使用 self 访问实例属性
        data = json.loads(v)
        ts = data['data'][0]['done_ts'] / 1000
        id = data['data'][0]['id']
        chunk_info = data['data'][0]['chunk_info']
        timeArray = time.localtime(ts)
        otherStyleTime = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        print(f"Received message with key: {k}, {otherStyleTime}, {id}, {chunk_info}")

    def run(self):
        # client = test_get_client_with_args('consumer1')
        client = get_client(
            group_id="mm.test.doc.10.bge",
            consumer_config={'auto.offset.reset': 'earliest',
                             'max.poll.interval.ms': 8000000,
                             'heartbeat.interval.ms': 3000,
                             'session.timeout.ms': 30000
                             }
        )
        # b = get_client()
        # print('start producing...')
        client.produce_message(topic='test-topic', value={'nihao': '1'})
        # client.produce_messages_in_batch(topic='test-topic', messages=[(f'key{i}', f'value{i}') for i in range(100)])

        # print('start consuming...')
        client.consume_messages(topics=["iuap_aip_service_topic_file_process"], handle_func=self.handle_message,
                                return_by_func=self.return_func)
        client.consume_messages_with_threads(topics=['iuap_aip_service_topic_file_process'], handle_func=self.handle_message, num_threads=10)


if __name__ == '__main__':
    myc = MyConsumer()
    myc.run()



'''
Author: Hexu
Date: 2022-04-08 17:13:36
LastEditors: Hexu
LastEditTime: 2022-09-16 16:43:37
FilePath: /iw-algo-fx/intelliw/functions/__init__.py
Description:  init
'''
from intelliw.functions.general import convert_data
from intelliw.functions.select_columns import select_columns
from intelliw.functions.data_filtering import filter_by_condition


class Const:
    split_function_list = ["convert_data",
                           "select_columns", "filter_by_condition"]
    feature_process = "feature_process"
    feature_process_param = "feature_process_param"
    timeseries_process = "timeseries_process"
    timeseries_param = "timeseries_param"
    timeseries_group_param = "group_param"



'''
Author: hexu
Date: 2021-10-28 19:45:01
LastEditTime: 2022-02-14 14:04:35
LastEditors: your name
Description: 
FilePath: /iw-algo-fx/intelliw/utils/__init__.py
'''
from intelliw.utils.util import *



#!/usr/bin/env python
# coding: utf-8

import json
from intelliw.utils import get_json_encoder
from intelliw.config import config

msg_id = 0


class ErrorCode:
    limit_exceeded = '013-501-015001'
    invalid_request = '013-501-015002'


def get_msg_id():
    global msg_id
    msg_id += 1
    return msg_id


####### return msg #########
class HealthCheckResponse:
    def __init__(self, code, typ, msg='', data=None):
        self.code = code
        self.typ = typ
        self.msg = msg
        self.data = data

    def _format(self):
        return {
            "extrainfo": {
                "status": self.code,
                "message": self.msg,
            },
            "data": self.data
        }

    def __str__(self):
        return json.dumps(self._format(), cls=get_json_encoder(), ensure_ascii=False)

    def __call__(self):
        return self._format()


class APIResponse:
    def __init__(self, code, typ, msg='', data=None):
        self.code = code
        self.typ = typ
        self.msg = msg
        self.data = data

    def _format(self):
        result = self.data or self.msg
        if self.code in [ErrorCode.invalid_request, ErrorCode.limit_exceeded] \
                and isinstance(result, str):
            return {'code': self.code, 'message': result}

        if config.API_EXTRAINFO:
            return {
                "extrainfo": {
                    "status": self.code,
                    "message": self.msg,
                },
                "data": result
            }
        else:
            return result

    def __str__(self):
        return json.dumps(self._format(), cls=get_json_encoder(), ensure_ascii=False)

    def __call__(self):
        return self._format()


err_invalid_validate_request = APIResponse(500, "validate", "无效验证请求")


class CommonResponse:
    def __init__(self, code, typ, msg='', data='', **kwargs):
        self.code = code
        self.typ = typ
        self.msg = msg
        self.data = data
        self.other = kwargs

    def _format(self):
        keymap = {
            "id": config.INSTANCE_ID,
            "serviceId": config.SERVICE_ID,
            "msgid": get_msg_id(),
            "token": config.TOKEN,
            "code": self.code,
            "type": self.typ,
            "message": self.msg,
            "data": self.data
        }
        return {**keymap, **self.other}

    def __str__(self):
        return json.dumps(self._format(), cls=get_json_encoder(), ensure_ascii=False)

    def __call__(self):
        return self._format()


ok_infer_online_status = CommonResponse(200, "infer", "status ok")
err_import_alg_invalid_path = CommonResponse(500, "importalg", "无效算法包")
err_import_alg_missing_algorithpy = CommonResponse(
    500, "importalg", "算法包必须包含: algorithm.py")
err_import_alg_missing_algorithyaml = CommonResponse(
    500, "importalg", "算法包必须包含: algorithm.yaml")
err_import_alg_invalid_algorithpy_format = CommonResponse(
    500, "importalg", "algorithm.py 内容不符合规范")
err_import_alg_invalid_algorithyaml_format = CommonResponse(
    500, "importalg", "algorithm.yaml 内容不符合规范")
err_missing_router_func = CommonResponse(
    500, "importalg", 'algorithm.yaml错误, 自定义api对应函数不存在')

err_import_model_missing_modelyaml = CommonResponse(
    500, "importmodel", "模型包必须包含: model.yaml")
err_import_alg_invalid_modelyaml_format = CommonResponse(
    500, "importmodel", "model.yaml 内容不符合规范")
err_import_model_invalid = CommonResponse(500, "importmodel", "模型参数文件无效")

ok_import_algorithm = CommonResponse(200, "importalg", "框架导入算法成功")
ok_import_model = CommonResponse(200, "importmodel", "框架导入模型成功")
err_import_algorithm = CommonResponse(500, "importalg", "框架导入算法失败")
err_import_model = CommonResponse(500, "importmodel", "框架导入模型失败")
err_transform_support_type = \
    CommonResponse(500, "importmodel",
                   "当前特征工程只支持四种类型，pre-train(训练前), pre-predict(推理前), pre-train(推理后)")

ok_train_finish = CommonResponse(200, 'train_finish', '', '')
err_empty_train_data = CommonResponse(500, "train_finish", "训练错误-训练数据为空")
err_empty_val_data = CommonResponse(500, "train_finish", "训练错误-验证数据为空")
err_train = CommonResponse(500, "train_finish", "训练错误-算法程序错误")
err_dataset = CommonResponse(500, "train_finish", '数据集存在问题')
err_function_process = CommonResponse(500, "train_finish", "特征值/数据处理错误")
err_streaming_not_supported = CommonResponse(500, "train_finish", "训练错误-未实现流式训练接口")


ok_custom_finish = CommonResponse(200, 'custom_finish', '', '')
err_custom_process = CommonResponse(500, "custom_finish", "自定义构建算法执行失败")

err_missing_algorithm_mod = CommonResponse(
    500, "importalg", "算法模块文件 algorithm.py 无效")
err_missing_algorithm_class = CommonResponse(
    500, "importalg", "算法模块文件没有定义定义类 Algorithm ")
err_missing_train_method = CommonResponse(
    500, "importalg", "算法模块文件类 Algorithm 缺少 train 函数")
err_missing_train_parameter = CommonResponse(
    500, "importalg", "算法类 Algorithm 函数 train 参数定义不正确")
err_missing_report_train_info_method = CommonResponse(
    500, "importalg", "算法模块文件类 Algorithm 缺少 report_train_info 函数")
err_missing_infer_method = CommonResponse(
    500, "importalg", "算法模块文件类 Algorithm 缺少 infer 函数")
err_missing_infer_parameter = CommonResponse(500,
                                             "importalg",
                                             "算法类 Algorithm 函数 infer 参数定义不正确， 应该为 infer(self, data)")
err_missing_init_method = CommonResponse(
    500, "importalg", "算法模块文件类 Algorithm 缺少 __init__ 函数")
err_missing_init_parameter = CommonResponse(500,
                                            "importalg",
                                            "算法类 Algorithm 函数 __init__ 参数定义不正确， 应该为 __init__(self, parameters)")
err_missing_load_method = CommonResponse(
    500, "importalg", "算法模块文件类 Algorithm 缺少 load 函数")
err_missing_load_parameter = CommonResponse(500,
                                            "importalg",
                                            "算法类 Algorithm 函数 load 参数定义不正确， 应该为 load(self, path)")
err_missing_save_method = CommonResponse(
    500, "importalg", "算法模块文件类 Algorithm 缺少 save 函数")
err_missing_save_parameter = CommonResponse(500,
                                            "importalg",
                                            "算法类 Algorithm 函数 save 参数定义不正确， 应该为 save(self, path)")



'''
Author: Hexu
Date: 2022-09-26 13:34:19
LastEditors: Hexu
LastEditTime: 2022-09-26 13:40:13
FilePath: /iw-algo-fx/intelliw/utils/database/schema.py
Description:
'''

from typing import List

from sqlalchemy import Table, Column


class SchemaUtils(object):
    """该类一定要和sqlalchemy.ext.declarative.api.DeclarativeMeta类一起被继承
    >>> from sqlalchemy.ext.declarative import declarative_base
    >>> from intelliw.utils.database.schema import SchemaUtils
    >>> Base = declarative_base()
    >>> class ShannonResearchReportFinalDistinct(Base, SchemaUtils):
    >>>     __tablename__ = 'shannon_research_report_final_distinct'
    >>>     desc = '研报去重数据'
    >>>     OBJECT_ID = Column(Integer, primary_key=True)
    >>>     FILEPATH = Column(String(500), doc={'zh': 'OSS路径'})
    >>>     FILENAME = Column(String(500), doc={'zh': '文件名'})
    """

    __table__ = None

    @classmethod
    def table(cls) -> Table:
        """返回这个orm对象对应的SQLAlchemy Table对象"""
        return cls.__table__

    @classmethod
    def tablename(cls) -> str:
        """返回这个orm对象对应数据库里的表名"""
        return str(cls.__table__)

    @classmethod
    def orm_name(cls) -> str:
        """返回这个orm对象对应的ORM类类名"""
        return cls.__name__

    @classmethod
    def columns(cls) -> List[Column]:
        """返回这个orm对象的所有SQLAlchemy Column列对象"""
        return list(cls.table().columns)

    @classmethod
    def column_names(cls) -> List[str]:
        """返回这个orm对象对应数据库里的列名"""
        return [column.name for column in cls.columns()]

    @classmethod
    def get_column(cls, column_name) -> Column or None:
        """返回这个orm对象和column_name同名的SQLAlchemy Column列对象"""
        for column in cls.columns():
            if column.name == column_name:
                return column
        raise ValueError(
            'Cannot find a column with name: {}！'.format(column_name))

    def get_value(self, column_name: str = None, column: Column = None):
        """返回这个orm对象和column_name列的值"""
        return getattr(self, column_name or column.name)

    def get_values(self):
        return [self.get_value(column_name) for column_name in self.column_names()]

    def __repr__(self):
        s = []
        for column_name in self.column_names():
            s.append('[{}={}]'.format(
                column_name, repr(self.get_value(column_name))))
        return '<{} {}>'.format(self.orm_name(), ', '.join(s))



'''
Author: hexu
Date: 2021-10-18 10:44:06
LastEditTime: 2023-05-10 15:32:22
LastEditors: Hexu
Description: 用来存储项目中需要的全局环境变量， 减少代码的耦合
FilePath: /iw-algo-fx/intelliw/utils/global_val.py
'''
import threading


class GlobalVal:
    # 单例锁
    _instance_lock = threading.Lock()
    _recorder = None
    _monitor = None
    _global_dict = {}

    def __new__(cls):
        """ 单例，防止调用生成更多环境变量dict """
        if not hasattr(GlobalVal, "_instance"):
            with GlobalVal._instance_lock:
                if not hasattr(GlobalVal, "_instance"):
                    GlobalVal._instance = object.__new__(cls)
        return GlobalVal._instance

    @staticmethod
    def set(key, value):
        """ 定义一个全局变量 """
        GlobalVal._global_dict[key] = value

    @staticmethod
    def set_dict(_dict: dict):
        if type(_dict) != dict:
            raise TypeError(
                "GlobalVal.set_dict input must be a dict, error type: ", type(_dict))
        for k, v in _dict.items():
            GlobalVal._global_dict[k] = v

    @staticmethod
    def get(key, default=None):
        """ 获得一个全局变量,不存在则返回默认值 """
        return GlobalVal._global_dict.get(key, default)

    @staticmethod
    def pop(key, default=None):
        """ 获得一个全局变量,并从全局变量中删除,不存在则返回默认值 """
        return GlobalVal._global_dict.pop(key, default)

    @staticmethod
    def value_is(key, value):
        return GlobalVal._global_dict.get(key, None) == value

    @staticmethod
    def has(key):
        return key in GlobalVal._global_dict

    @staticmethod
    def clear():
        del GlobalVal._global_dict
        GlobalVal._global_dict = {}

    def __getattr__(self, key):
        if key == "recorder":
            return GlobalVal._recorder
        elif key == "monitor":
            return GlobalVal._monitor
        return GlobalVal._global_dict.get(key)

    def __setattr__(self, key, value):
        if key == "recorder":
            GlobalVal._recorder = value
        elif key == "monitor":
            GlobalVal._monitor = value
        else:
            GlobalVal._global_dict[key] = value


gl = GlobalVal()



class IntelliwDict(dict):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.get_dict = None


class Request:
    """
    intelliw server request
    """

    def __init__(self, **kwargs) -> None:
        self.header = None
        self.json = ""
        self.query = {}
        self.form = IntelliwDict()
        self.files = {}
        self.body = ""
        self.batch_params = {}
        self.context = None
        self.raw = None
        self.method = "GET"
        self.url = None

        for k, v in kwargs.items():
            setattr(self, k, v)



import json
import os
import subprocess as sb
import sys
import time
from queue import Queue, Empty
from threading import Thread
from intelliw.core.pipeline import Pipeline, Const
from intelliw.utils import message
from intelliw.utils.logger import _get_framework_logger

logger = _get_framework_logger()


class StarterMode:
    CMD = 'cmd'


class ProcedureType:
    INFER = 'infer'
    TRAIN = 'train'
    IMPORT_ALG = 'importalg'
    IMPORT_MODEL = 'importmodel'
    IMPORT = [IMPORT_ALG, IMPORT_MODEL]


class CustomError(Exception):
    def __init__(self, msg) -> None:
        self.msg = msg

    def __str__(self) -> str:
        return '自定义算法异常, {}'.format(self.msg)


class Custom:
    """
    Custom entrypoint
    """

    def __init__(self, path, reporter_addr=None, procedure_type=None, starter_mode=StarterMode.CMD):
        """
        init custom entrypoint
        args:
            path -- algorithm's path
            procedure_type -- custom algorithm procedure type (infer/train)
            starter_mode -- custom algorithm start mode (default: CMD)
        """
        self.algorithm_path = path
        self.starter_mode = starter_mode
        self.procedure_type = procedure_type
        self.pipeline = Pipeline(reporter_addr)
        self.report = self.pipeline.recorder.report
        self.is_import = procedure_type in ProcedureType.IMPORT
        self.config = self.load_config(self.algorithm_path, self.is_import)

    def load_config(self, path, is_import=False):
        if is_import:
            return {}
        yaml_path = os.path.join(path, Const.model_yaml)
        if not os.path.exists(yaml_path):
            raise CustomError(f"加载配置文件失败，配置文件不存在{yaml_path}")
        config = self.pipeline.load_config(yaml_path)
        if not config or not config.get("Model", False):
            raise CustomError("加载配置文件失败，配置文件为空")
        return config["Model"]

    def start_engine(self):
        # 初始化系统环境变量
        self.pipeline.initialize_env(
            self.config.get("system")
        )
        # 获取启动方式, 目前就命令行方式
        if self.starter_mode == StarterMode.CMD:
            return self._get_command()
        return self._get_command()

    def format_msg(self, msg: message.CommonResponse):
        msg.other = {'businessType': self.procedure_type}
        return msg

    def run_import(self):
        # 项目初始化成功
        if self.is_import:
            self.report(message.ok_import_algorithm)
        else:
            self.report(message.ok_import_model)

    def run_process(self):
        try:
            starter = self.start_engine()
            self.report(message.ok_import_model)
            logger.info(f"\033[33mCustom Mode:\033[0m {starter}")
            starter.run()
        except Exception as e:
            # 项目运行失败
            self.pipeline.raise_exception(
                self.format_msg(message.err_custom_process), e
            )
        # 项目运行成功
        self.report(self.format_msg(message.ok_custom_finish))

    def run(self):
        if self.procedure_type in ProcedureType.IMPORT:
            # 导入模式
            self.run_import()
        else:
            # 运行模式
            self.run_process()

    def _get_command(self):
        # 获取command
        command = self.config.get('boot', [])
        # 获取算法环境变量
        parameters = self.config.get('parameters', {})
        os.environ['PARAMETERS'] = json.dumps(parameters, ensure_ascii=False)
        return CMD(command, self.algorithm_path)


class CMD:
    def __init__(self, cmd, path=None):
        self.cmd = [str(c) for c in cmd]
        self.path = path
        self.out_queue = Queue(100000)
        self.err_queue = Queue(10000)

    def enqueue_output(self, out, _type):
        _queue = self.err_queue if _type == "err" else self.out_queue
        for line in iter(out.readline, b''):
            if _queue.full():
                _queue.get()
            _queue.put(line)
        out.close()

    def shell_mode(self):
        if len(self.cmd) > 2:
            if self.cmd[1] == '-c':
                if 'bash' in self.cmd[0] or 'sh' in self.cmd[0]:
                    self.cmd = " ".join(self.cmd)
                    return True
        return False

    def run(self):
        if not self.cmd:
            raise CustomError("启动命令为空")
        ON_POSIX = 'posix' in sys.builtin_module_names
        shell = self.shell_mode()

        res = sb.Popen(self.cmd, stdout=sb.PIPE, stderr=sb.PIPE, env=os.environ,
                       cwd=self.path, close_fds=ON_POSIX, shell=shell)

        out_thread = Thread(target=self.enqueue_output, args=(res.stdout, 'out'), daemon=True)
        out_thread.start()

        err_thread = Thread(target=self.enqueue_output, args=(res.stderr, 'err'), daemon=True)
        err_thread.start()

        err_msg = ""
        while True:
            # out
            try:
                line = self.out_queue.get_nowait()
            except Empty:
                time.sleep(0.01)
            else:
                logger.info(line.decode('utf-8').strip('\n'))

            # err
            try:
                err_line = self.err_queue.get_nowait()
            except Empty:
                time.sleep(0.01)
            else:
                err_msg = err_line.decode('utf-8').strip('\n')
                logger.error(err_msg)

            if sb.Popen.poll(res) is not None:
                if res.returncode > 0:
                    break
                return None

        # 程序异常 跳出循环
        try:
            err_msg = res.stderr.read().decode('utf-8')
        except:
            pass
        raise CustomError(err_msg)

    def __str__(self):
        return f"\033[33mCMD\033[0m - {self.cmd}"





import base64
import hashlib
import math
import os
import sys
import time
import requests

import jwt


def register_service(code, port, keepcode='false'):
    url = f"{os.environ['domain_url']}/iuap-aip-console/self/gateway/expplat/srvpath"
    data = {
        "srvid": code,
        "srvips": [f"{os.environ['local_ip']}:{port}"],
        "responseTimeout": 300000,
        "requsert"
        "keepCode": keepcode.lower() == 'true'
    }
    header = {"X-tenantId": "0"}
    sign(url, data, header)
    resp = requests.post(url=url, json=data, headers=header)
    print(resp.text)
    resp.raise_for_status()


def sign(url: str, params: dict, headers: dict):
    params['AuthSdkServer'] = "true"
    headers['YYCtoken'] = sign_authsdk(url, params)
    return headers, params


def sign_authsdk(url: str, params: dict, ak=None, ase=None) -> str:
    """
    generate iuap signature

    :param url: request url, without parameters
    :param params:  request parameters, x-www-form-urlencoded request's body parameters should also be included.
    :param ak: ACCESS KEY
    :param ase: ACCESS SECRET
    :return: iuap signature
    """
    if not ak:
        ak = os.environ['ACCESS_KEY']
    if not ase:
        ase = os.environ['ACCESS_SECRET']

    issue_at = __issue_at()
    sign_key = __build_sign_key(ak, ase, issue_at, url)
    jwt_payload = {
        "sub": url,
        "iss": ak,
        "iat": issue_at
    }
    if params is not None and len(params) > 0:
        sorted_params = sorted(params.items())
        for item in sorted_params:
            if item[1] is None:
                val = ''
            elif len(str(item[1])) >= 1024:
                val = str(__java_string_hashcode(str(item[1])))
            else:
                val = str(item[1])
            jwt_payload[item[0]] = val

    jwt_token = jwt.encode(jwt_payload, key=sign_key, algorithm='HS256')
    return jwt_token if isinstance(jwt_token, str) else jwt_token.decode('utf-8')


def __issue_at():
    issue_at = int(time.time())
    issue_at = math.floor(issue_at / 600) * 600
    return issue_at


def __build_sign_key(access_key, access_secret, access_ts, url):
    str_key = access_key + access_secret + str(access_ts * 1000) + url
    sign_key_bytes = hashlib.sha256(str_key.encode('UTF-8')).digest()
    return base64.standard_b64encode(sign_key_bytes).decode('UTF-8')


def __java_string_hashcode(s: str):
    h = 0
    for c in s:
        h = (31 * h + ord(c)) & 0xFFFFFFFF
    return ((h + 0x80000000) & 0xFFFFFFFF) - 0x80000000


if __name__ == '__main__':
    args = sys.argv
    # print(args)
    register_service(args[1], args[2], args[3])



#!/usr/bin/env python
# coding: utf-8
from intelliw.config import config

config.update_by_env()

from intelliw.interface.controller import FrameworkArgs, __parse_args
from intelliw.utils.logger import _get_framework_logger
from intelliw.interface.iwapi.iwapp import ApiService

logger = _get_framework_logger()
framework_args = FrameworkArgs(__parse_args())

config.FRAMEWORK_MODE = "infer"
api_service = ApiService(framework_args.port, framework_args.path, framework_args.response)

if __name__ == '__main__':
    api_service.run()



import cv2
import numpy as np

KEY_FLAG = "flag"
KEY_INTERPOLATION = "interpolation"
KEY_WIDTH = "width"
KEY_HEIGHT = "height"


def im_decode(data, params):
    if type(data) is not np.ndarray:
        data = np.fromstring(data, np.uint8)
    flag = cv2.IMREAD_GRAYSCALE
    if KEY_FLAG in params:
        flag = params[KEY_FLAG]
    img = cv2.imdecode(data, flag)
    return img


def im_resize(data, params):
    if type(data) is not np.ndarray:
        raise Exception(
            "expected input data type numpy.ndarray in func im_resize")
    img_width = 0
    img_height = 0
    interpolation = cv2.INTER_LINEAR
    if KEY_WIDTH in params:
        img_width = params[KEY_WIDTH]
    else:
        raise Exception("expected param width in opencv func im_resize")
    if KEY_HEIGHT in params:
        img_height = params[KEY_HEIGHT]
    else:
        raise Exception("expected param height in opencv func im_resize")
    if KEY_INTERPOLATION in params:
        interpolation = params[KEY_INTERPOLATION]
    img = cv2.resize(data, (img_width, img_height),
                     interpolation=interpolation)
    return img


def im_transpose(data, params):
    if type(data) is not np.ndarray:
        raise Exception(
            "expected input data type numpy.ndarray in func im_transpose")
    img_width = 0
    img_height = 0
    if KEY_WIDTH in params:
        img_width = params[KEY_WIDTH]
    else:
        raise Exception("expected param width in opencv func im_transpose")
    if KEY_HEIGHT in params:
        img_height = params[KEY_HEIGHT]
    else:
        raise Exception("expected param height in opencv func im_transpose")
    img = cv2.transpose(data, (img_height, img_width))
    return img



import base64
import os
import time
import traceback
import logging
import requests
from intelliw.utils.iuap_request import sign_authsdk

logger = logging.getLogger('yms_download')
logger.setLevel("INFO")
logger_code = "intelliw.logger.level"


def set_env(k, v):
    if k not in os.environ:
        os.environ[k] = str(v)


class YmsSysEnv:
    YMS_ENV_CONFIG_PATH = './yms_env_config.yaml'

    def __init__(self):
        self.init_config()
        self.special_env_map = {}
        self.mid_info_map = {}
        self.is_premises = False
        self.local_cache = None

    def init_config(self):
        self.ACCESS_KEY = os.getenv('ACCESS_KEY') or os.getenv('access.key')
        self.ACCESS_SECRET = os.getenv('ACCESS_SECRET') or os.getenv('access.secret')
        self.YMS_CONSOLE_ADDRESS = os.getenv('YMS_CONSOLE_ADDRESS') or os.getenv('disconf.conf_server_host')
        self.YMS_CONSOLE_ACTIVE = os.getenv('YMS_CONSOLE_ACTIVE') or os.getenv('mw_profiles_active')
        self.YMS_CONSOLE_APP_CODE = os.getenv('YMS_CONSOLE_APP_CODE', 'iuap-aip-console')
        self.YMS_CONFIG_FIlE_ADDRESS = f"{self.YMS_CONSOLE_ADDRESS}/api/v2/config/file"
        self.YMS_PW_DECODE_ADDRESS = f"{self.YMS_CONSOLE_ADDRESS}/api/v1/ymsConfig/enc/decValue"

    def _env_precess(self, env):
        for config_group in env['ymsConfigGroupVos']:
            for config in config_group['configItems']:
                code = config['code']
                value: str = config['value']
                if value.find('#{') >= 0:
                    self.special_env_map[code] = value
                    continue
                if value.startswith('YMS(') and value.endswith(')'):
                    value = decode_yms_pw(value)

                if code == logger_code:
                    value = os.environ.get(logger_code) or value

                if code == "isPremises":
                    self.is_premises = value == 'true'
                try:
                    set_env(code, value)
                    self.mid_info_map[code] = value
                except Exception as e:
                    logger.error("set yms environ error: {}, Key:{}, Value:{}".format(e, code, value))

                if self.local_cache is not None:
                    self.local_cache.write(f"{code}={value}\n")

    def _generate_value(self, result: str):
        if result.find('#{') >= 0:
            for k, v in self.mid_info_map.items():
                variable = '#{' + k + '#}'
                if result.find(variable) >= 0:
                    result = result.replace(variable, v)
        return result

    def init_children_info(self, children):
        for c in children:
            if c.get('ymsConfigGroupVos'):
                self._env_precess(c)

            if c.get('children'):
                self.init_children_info(c['children'])

    def _get_local_cache(self):
        if self.local_cache is None and not os.path.exists(self.YMS_ENV_CONFIG_PATH):
            self.local_cache = open(self.YMS_ENV_CONFIG_PATH, "w")
        return self.local_cache

    def close_file(self):
        if self.local_cache is not None and not self.local_cache.closed:
            self.local_cache.close()
            self.local_cache = None

    def init_envs(self, env):
        self._get_local_cache()

        if not env:
            logger.warning('YMS配置为空')
            return

        self._env_precess(env['data'])

        if env['data'].get('children'):
            self.init_children_info(env['data']['children'])

        for code, value in self.special_env_map.items():
            value = self._generate_value(value)

            try:
                set_env(code, value)
                self.mid_info_map[code] = value
            except Exception as e:
                logger.error("set yms environ error: {}, Key:{}, Value:{}".format(e, code, value))

            if self.local_cache is not None:
                self.local_cache.write(f"{code}={value}\n")

    def init_redis(self, redis):
        if not redis:
            return

        for c in redis['clients']:
            pool_name = c['pool']
            database = c.get('database')
            redis_code = c['name']

            pool_info = None
            for p in redis['pools']:
                if p['name'] == pool_name:
                    pool_info = p

            if pool_info:
                if pool_info['type'] == 'single':
                    set_env(f'{redis_code}_REDIS_URL', f"{pool_info['host']}:{pool_info['port']}")
                    if self.local_cache is not None:
                        self.local_cache.write(
                            f"{redis_code}_REDIS_URL={pool_info['host']}:{pool_info['port']}\n")
                else:
                    set_env(f'{redis_code}_REDIS_URL', pool_info['nodes'])
                    if self.local_cache is not None:
                        self.local_cache.write(f"{redis_code}_REDIS_URL={pool_info['nodes']}\n")

                value = decode_yms_pw(pool_info.get('password', ''))
                sentinel_value = decode_yms_pw(pool_info.get('sentinelPassword', ''))
                set_env(f'{redis_code}_REDIS_TYPE', f"{pool_info.get('type', '')}")
                set_env(f'{redis_code}_REDIS_DATABASE', f'{database}')
                set_env(f'{redis_code}_REDIS_PASSWORD', value)
                set_env(f'{redis_code}_REDIS_SENTINEL_PASSWORD', sentinel_value)
                set_env(f'{redis_code}_REDIS_MASTER_NAME', f"{pool_info.get('masterName', '')}")
                set_env(f'{redis_code}_REDIS_POOL_SIZE', f"{pool_info.get('max-idle', '')}")

                if self.local_cache is not None:
                    self.local_cache.write(f"{redis_code}_REDIS_TYPE={pool_info.get('type', '')}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_DATABASE={database}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_PASSWORD={value}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_SENTINEL_PASSWORD={sentinel_value}\n")
                    self.local_cache.write(
                        f"{redis_code}_REDIS_MASTER_NAME={pool_info.get('masterName', '')}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_POOL_SIZE={pool_info.get('max-idle', '')}\n")

    def __del__(self):
        self.close_file()


yms_sys_env = YmsSysEnv()


def request_yms(url, method="GET", params=None, data=None):
    for i in range(1, 5):
        try:
            token = sign_authsdk(url, params, yms_sys_env.ACCESS_KEY, yms_sys_env.ACCESS_SECRET)
            headers = {
                'Content-Type': 'application/json',
                'YYCtoken': token,
                'dcAddr': base64.b64encode(bytes(url, 'utf-8')).decode()
            }
            resp = requests.request(
                method=method, url=url, params=params,
                json=data, verify=False, headers=headers, timeout=5.0
            )
            resp.raise_for_status()
            return resp.json()
        except requests.exceptions.RequestException as e:
            if i == 4:
                raise e
            time.sleep(i * 2)
            try:
                body = e.response.text if hasattr(e.response, "text") else e.response
            except:
                body = ""
            logger.error(
                "request retry time: %s, url: %s, body: %s, error: %s",
                i, url, body, e)


def decode_yms_pw(pw):
    resp = request_yms(yms_sys_env.YMS_PW_DECODE_ADDRESS,
                       method='POST',
                       data=[pw, ])

    if resp['success'] != 'true':
        logger.error(f"{resp['error_code']}:{resp['error_message']}")
        return ""

    data = resp.get('data', {})
    return data.get(pw, "")


def download_ca_file():
    from intelliw.utils.yaml_downloader import download
    workdir = os.getenv('WORKDIR', '/root')
    ca_path = os.path.join(workdir, 'cert')
    os.makedirs(ca_path, exist_ok=True)
    ca_url = os.getenv('CA_FILE_URL')
    if os.getenv('useGpaas') == 'true':
        ca_url = f"{os.environ['domain.url']}{ca_url}"
    try:
        download(ca_url, ca_path, True)
        logger.info("下载证书成功")
    except Exception as e:
        logger.error(f"下载证书失败: {e}, 证书下载地址: {ca_url}")


def run():
    try:
        yms_sys_env.init_config()
        if not yms_sys_env.YMS_CONSOLE_ADDRESS:
            logger.warning("未配置YMS_CONSOLE_ADDRESS，不进行YMS配置拉取")
            return

        resp = request_yms(yms_sys_env.YMS_CONFIG_FIlE_ADDRESS,
                           params={
                               'app': yms_sys_env.YMS_CONSOLE_APP_CODE,
                               'env': yms_sys_env.YMS_CONSOLE_ACTIVE}
                           )
        if resp.get('success') == 'false':
            logger.error(f"{resp['error_code']}:{resp['error_message']}")
            return

        yms_sys_env.init_envs(resp.get('environment'))
        yms_sys_env.init_redis(resp.get('redis'))
        logger.info("执行YMS配置拉取成功")

        download_ca_file()

    except Exception as e:
        err_msg = traceback.format_exc()
        logger.error("yms配置下载失败: %s, 详细信息: %s", e, err_msg)
    finally:
        try:
            yms_sys_env.close_file()
        except:
            pass



import base64

KEY_FIELD = "field"
KEY_BS64DECODE = "bs64decode"


def get_value(data, params):
    # get bs64 string
    if KEY_FIELD not in params:
        raise Exception("expected param field in general func get_value")
    field = params[KEY_FIELD]
    value_list = data[field]

    # turn bs64 string into numpy array
    if KEY_BS64DECODE in params and params[KEY_BS64DECODE]:
        # bs64_str = unquote(bs64_str)
        bs64_str = base64.b64decode("".join(value_list))
        return bs64_str
    else:
        return value_list


def convert_data(data, params):
    print("do convert_data")
    return data



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   yms_downloader_v2.py
@Time    :   2025/07/14 15:36:25
@Author  :   zhangbin01
@Version :   1.0
@Contact :   zhangbin01@yonyou.com
@description : 业务逻辑 https://uap-wiki.yyrd.com/pages/viewpage.action?pageId=261987779
'''


import base64
import os
import time
import traceback
import logging
import requests
import json
from urllib.parse import urlparse
from collections import defaultdict
from intelliw.utils.iuap_request import sign_authsdk
from intelliw.utils.iuap_request import get_module_config
from intelliw.utils.yms_encrypt_util import YmsEncryptUtil
from intelliw.utils.yms_config_util import parse_replace_config
from intelliw.utils.util import parse_host_port_from_netloc

from intelliw.utils.logger import _get_framework_logger

logger = _get_framework_logger()

logger_code = "intelliw.logger.level"



def set_env(k, v):
    if k not in os.environ:
        os.environ[k] = str(v)


class YmsSysEnv:
    # 配置文件路径增加时间格式后缀, 便于每次重启生成新的yaml文件,排查问题
    time_prefix = time.strftime('%Y%m%d%H%M%S', time.localtime())
    YMS_ENV_CONFIG_PATH = './yms_env_config_{}.yaml'.format(time_prefix)
    YMS_ENV_CONFIG_NEW_PATH = './yms_env_config_new_{}.yaml'.format(time_prefix)
    YMS_ENV_RESP_CONFIG_FILE = './yms_env_resp_config_{}.json'.format(time_prefix)
    YMS_ENV_RESP_CONFIG_NEW_FILE = './yms_env_resp_config_new_{}.json'.format(time_prefix)
    YMS_ENV_LOCAL_YONBIP_CONFIG_DIR = '/app/config/global/'
    YMS_YONBIP_CONFIG_FN = 'yonbip_config.json'
    YMS_ENV_LOCAL_YMS_MIDDLEWARE_CONFIG_DIR = '/app/config/middleware/'
    YMS_MIDDLEWARE_CONFIG_FN = 'yms_middleware.json'
    # YMS_ENV_LOCAL_MODULE_CONFIG_DIR = '/app/config/global/'
    YMS_MODULE_CONFIG_FN = 'module_config.json'

    DB_FIELDS = [
        "tenantList",
        "validationQuery",
        "defaultCatalog",
        "logAbandoned",
        "password",
        "maxIdle",
        "testWhileIdle",
        "ts",
        "removeAbandoned",
        "removeAbandonedTimeout",
        "defaultAutoCommit",
        "testOnConnect",
        "minIdle",
        "initialSize",
        "maxWait",
        "dbType",
        "url",
        "testOnBorrow",
        "minEvictableIdleTimeMillis",
        "timeBetweenEvictionRunsMillis",
        "testOnReturn",
        "driverClassName",
        "maxActive",
        "username",
        "validationQueryTimeout",
        "name",
        "lastUpdateTime",
   ]

    def __init__(self):
        self.init_config()
        self.yms_enc_tools = YmsEncryptUtil()
        self.special_env_map = {}
        self.mid_info_map = {}
        self.is_premises = False
        self.local_cache = None
        self.local_cache_new = None
        self.clients = {}

    def init_config(self):
        self.ACCESS_KEY = os.getenv('ACCESS_KEY') or os.getenv('access.key') or os.getenv('cf_clientAccessKey')
        self.ACCESS_SECRET = os.getenv('ACCESS_SECRET') or os.getenv('access.secret') or os.getenv('cf_clientAccessSecret')
        self.YMS_CONSOLE_ADDRESS = os.getenv('YMS_CONSOLE_ADDRESS') or os.getenv('disconf.conf_server_host')
        self.YMS_CONSOLE_ACTIVE = os.getenv('YMS_CONSOLE_ACTIVE') or os.getenv('mw_profiles_active')
        self.YMS_CONSOLE_APP_CODE = os.getenv('YMS_CONSOLE_APP_CODE', 'iuap-aip-console')
        self.YMS_ALGVEC_APP_CODE = os.getenv('YMS_ALGVEC_APP_CODE', 'iuap-aip-algvec')
        self.YMS_CONFIG_FILE_ADDRESS = f"{self.YMS_CONSOLE_ADDRESS}/api/v2/config/file"
        self.YMS_CONFIG_NEW_FILE_ADDRESS = f"{self.YMS_CONSOLE_ADDRESS}/api/v2/config/new/file"
        self.YMS_CONFIG_MODULE_URL = os.getenv('YMS_CONFIG_MODULE_URL', '/self/rest/api/moduleConfig')
        self.CONFIG_HOME = os.getenv('CONFIG_HOME')
        self.YMS_ENV_LOCAL_YONBIP_CONFIG_PATH = f'{self.YMS_ENV_LOCAL_YONBIP_CONFIG_DIR}{self.YMS_YONBIP_CONFIG_FN}' \
            if self.CONFIG_HOME is None \
            else f'{self.CONFIG_HOME}/global/{self.YMS_YONBIP_CONFIG_FN}'
        
        self.YMS_ENV_LOCAL_YMS_MIDDLEWARE_CONFIG_PATH = f'{self.YMS_ENV_LOCAL_YMS_MIDDLEWARE_CONFIG_DIR}{self.YMS_MIDDLEWARE_CONFIG_FN}' \
            if self.CONFIG_HOME is None \
            else f'{self.CONFIG_HOME}/middleware/{self.YMS_MIDDLEWARE_CONFIG_FN}'
        
        self.AI_CONSOLE_MODULE = os.getenv('AI_CONSOLE_MODULE') # AI_CONSOLE_MODULE的值是${domain.iuap-aip-console}/self/rest/api/moduleConfig（完整请求地址）

        self.useGpaas = 'true' == os.environ.get('useGpaas', 'false')

    def _env_precess(self, env):
        for config_group in env['ymsConfigGroupVos']:
            for config in config_group['configItems']:
                code = config['code']
                value: str = config['value']
                if value.find('#{') >= 0:
                    self.special_env_map[code] = value
                    continue
                if value.startswith('YMS(') and value.endswith(')'):
                    # value = decode_yms_pw(value)
                    value = self.yms_enc_tools.decrypt(value)

                if code == logger_code:
                    value = os.environ.get(logger_code) or value

                if code == "isPremises":
                    self.is_premises = value == 'true'
                try:
                    set_env(code, value)
                    self.mid_info_map[code] = value
                except Exception as e:
                    logger.error("set yms environ error: {}, Key:{}, Value:{}".format(e, code, value))

                if self.local_cache is not None:
                    self.local_cache.write(f"{code}={value}\n")

    def _env_read_kv(self, env, raw_dict):
        for  key, value in env.items():
            if key == '_meta':
                continue
            if value is not None and value.startswith('YMS(') and value.endswith(')'):
                value = self.yms_enc_tools.decrypt(value)

            if key == logger_code:
                value = os.environ.get(logger_code) or value

            if key == "isPremises":
                self.is_premises = value == 'true'
            raw_dict[key] = value

    def _env_precess_new(self, raw_dict, env):
        for  key, value in env.items():
            if key == '_meta':
                continue
            if value is not None and value.startswith('YMS(') and value.endswith(')'):
                value = self.yms_enc_tools.decrypt(value)

            if key == logger_code:
                value = os.environ.get(logger_code) or value

            if key == "isPremises":
                self.is_premises = value == 'true'
            raw_dict[key] = value
        
        done_dict = parse_replace_config(raw_dict)
        if done_dict is None:
            return
        for key, value in done_dict.items():
            try:
                set_env(key, value)
            except Exception as e:
                logger.error("set yms environ error: {}, Key:{}, Value:{}".format(e, key, value))

            if self.local_cache_new is not None:
                self.local_cache_new.write(f"{key}={value}\n")

    def _generate_value(self, result: str):
        if result.find('#{') >= 0:
            for k, v in self.mid_info_map.items():
                variable = '#{' + k + '#}'
                if result.find(variable) >= 0:
                    result = result.replace(variable, v)
        return result

    def init_children_info(self, children):
        for c in children:
            if c.get('ymsConfigGroupVos'):
                self._env_precess(c)

            if c.get('children'):
                self.init_children_info(c['children'])

    def _get_local_cache(self):
        if self.local_cache is None and not os.path.exists(self.YMS_ENV_CONFIG_PATH):
            self.local_cache = open(self.YMS_ENV_CONFIG_PATH, "w")
        return self.local_cache

    def _get_local_cache_new(self):
        if self.local_cache_new is None and not os.path.exists(self.YMS_ENV_CONFIG_NEW_PATH):
            self.local_cache_new = open(self.YMS_ENV_CONFIG_NEW_PATH, "w")
        return self.local_cache_new

    def close_file(self):
        if self.local_cache is not None and not self.local_cache.closed:
            self.local_cache.close()
            self.local_cache = None

    def close_file_new(self):
        if self.local_cache_new is not None and not self.local_cache_new.closed:
            self.local_cache_new.close()
            self.local_cache_new = None

    def init_envs(self, env):
        self._get_local_cache()

        if not env:
            logger.warning('YMS配置为空')
            return

        self._env_precess(env['data'])

        if env['data'].get('children'):
            self.init_children_info(env['data']['children'])

        for code, value in self.special_env_map.items():
            value = self._generate_value(value)

            try:
                set_env(code, value)
                self.mid_info_map[code] = value
            except Exception as e:
                logger.error("set yms environ error: {}, Key:{}, Value:{}".format(e, code, value))

            if self.local_cache is not None:
                self.local_cache.write(f"{code}={value}\n")


    def flat_clients(self):
        '''
        扁平化clients配置
        :return: results
        '''
        results = defaultdict(set)
        if not self.clients:
            logger.warning('YMS新的 clients配置为空')
            self.clients = {}
            return results
        for key in self.clients:
            for client_key in self.clients[key]:
                if isinstance(self.clients[key][client_key], list):
                    for client_name in self.clients[key][client_key]:
                        results[client_key].add(client_name)
                else:
                    logger.warning(f"YMS新的 clients配置格式错误 client_key: {client_key}, type:{self.clients[key][client_key]}")
        
        return results


    def init_envs_new(self, env):
        '''
        初始化环境变量
        :param env:
        :return:
        '''

        if not env:
            logger.warning('YMS新配置为空')
            return

        raw_dict = {}
        for config_key in env:
            if config_key.startswith("config."):
                self._env_read_kv(env[config_key], raw_dict)
            elif config_key.startswith("client."):
                self.clients[config_key] = env[config_key]
            else:
                pass
                # (f'key: {config_key}, value: {env[config_key]}')
        
        replaced_dict = parse_replace_config(raw_dict)

        if replaced_dict is None:
            return
        for key, value in replaced_dict.items():
            try:
                set_env(key, value)
            except Exception as e:
                logger.error("set yms environ error: {}, Key:{}, Value:{}".format(e, key, value))

            if self.local_cache_new is not None:
                self.local_cache_new.write(f"{key}={value}\n")

    def init_connection_pools_new(self, middleware, connection_pool_codes):
        '''
        初始化连接池
        :param middleware:
        :param connection_pool_codes:
        :return:
        '''
        if not middleware or not connection_pool_codes:
            return
        connection_pools = middleware.get('connectionPools')

        datasources = middleware.get('datasources')
        for connection_code in connection_pool_codes:
            datasource = datasources.get(connection_code)
            if not datasource:
                logger.warning(f'connection_code: {connection_code} datasource not found')
                continue
            pools = datasource.get('pools')
            routeEnable = datasource.get('routeEnable')
            defaultSchema = datasource.get('defaultSchema')
            defaultDs = datasource.get('defaultDs')
            translateEnable = datasource.get('translateEnable')
            translateStrict = datasource.get('translateStrict')

            # 处理连接池 begin ===========================
            if pools is None:
                logger.warning(f'connection_code: {connection_code} pools not found')
                continue

            for pool_name in pools:
                try:
                    connection_info = connection_pools.get(pool_name)
                    if not connection_info:
                        logger.warning(f'connection_code: {connection_code}, pool_name: {pool_name} connection_info not found')
                        continue
                    # 生成连接信息 
                    # 如果 为defaultDs, 则使用connection_code， 否则使用 {connection_code}_{pool_name}
                    pre_connection_code = connection_code if pool_name == defaultDs else f'{connection_code}_{pool_name}'
                    schema_key = f'{pre_connection_code}_DB_DEFAULT_SCHEMA'
                    set_env(schema_key, defaultSchema)
                    if self.local_cache_new is not None:
                        self.local_cache_new.write(f"{schema_key}={defaultSchema}\n")
                    for db_field in self.DB_FIELDS:
                        key = f'{pre_connection_code}_DB_{db_field.upper()}'
                        value = connection_info.get(db_field)
                        if db_field == 'password':
                            value = self.yms_enc_tools.decrypt( connection_info.get('password'))
                        elif db_field == 'url' and value is not None:
                            if value.startswith('jdbc:'):
                                parsed_url = urlparse(value[5:])
                                host, port = parse_host_port_from_netloc(parsed_url.netloc)
                                set_env(f'{pre_connection_code}_DB_HOST', host)
                                set_env(f'{pre_connection_code}_DB_PORT', port)
                                if self.local_cache_new is not None:
                                    self.local_cache_new.write(f"{pre_connection_code}_DB_HOST={host}\n")
                                    self.local_cache_new.write(f"{pre_connection_code}_DB_PORT={port}\n")
                        set_env(key, value)
                        if self.local_cache_new is not None:
                            self.local_cache_new.write(f"{key}={value}\n")
                except Exception as e:
                    logger.warning(f'connection_code: {connection_code}, pool_name: {pool_name} error: {e}')
                # 处理连接池 end ===========================

    def init_redis_new(self, middleware, redis_codes):
        '''
        初始化redis
        :param middleware:
        :param redis_codes:
        :return:
        '''
        if not middleware or not redis_codes:
            return
        redis_pools = middleware.get('redis.pools')
        redis_regions = middleware.get('redis.regions')

        for redis_code in redis_codes:
            try:
                redis_region = redis_regions[redis_code]
                pool_name = redis_region.get('pool')
                mode = redis_region.get('mode')
                tenantSensitive = redis_region.get('tenantSensitive')
                namespace = redis_region.get('namespace')
                lastUpdateTime = redis_region.get('lastUpdateTime')
                database = redis_region.get('database')

                pool_info = redis_pools.get(pool_name)
                if not pool_info:
                    continue

                if pool_info['type'] == 'single':
                    set_env(f'{redis_code}_REDIS_URL', f"{pool_info['host']}:{pool_info['port']}")
                    set_env(f'{redis_code}_REDIS_HOST', f"{pool_info['host']}")
                    set_env(f'{redis_code}_REDIS_PORT', f"{pool_info['port']}")

                    if self.local_cache_new is not None:
                        self.local_cache_new.write(f"{redis_code}_REDIS_URL={pool_info['host']}:{pool_info['port']}\n")
                        self.local_cache_new.write(f"{redis_code}_REDIS_HOST={pool_info['host']}\n")
                        self.local_cache_new.write(f"{redis_code}_REDIS_PORT={pool_info['port']}\n")
                else:
                    set_env(f'{redis_code}_REDIS_URL', pool_info['nodes'])
                    if self.local_cache_new is not None:
                        self.local_cache_new.write(f"{redis_code}_REDIS_URL={pool_info['nodes']}\n")

                value = self.yms_enc_tools.decrypt(pool_info.get('password', ''))
                # sentinel_value = decode_yms_pw(pool_info.get('sentinelPassword', ''))
                sentinel_value = self.yms_enc_tools.decrypt(pool_info.get('sentinelPassword', ''))
                set_env(f'{redis_code}_REDIS_TYPE', f"{pool_info.get('type', '')}")
                set_env(f'{redis_code}_REDIS_DATABASE', f'{database}')
                set_env(f'{redis_code}_REDIS_MODE', f'{mode}')
                set_env(f'{redis_code}_REDIS_PASSWORD', value)
                set_env(f'{redis_code}_REDIS_SENTINEL_PASSWORD', sentinel_value)
                set_env(f'{redis_code}_REDIS_MASTER_NAME', f"{pool_info.get('masterName', '')}")
                set_env(f'{redis_code}_REDIS_POOL_SIZE', f"{pool_info.get('maxIdle', '')}")

                if self.local_cache_new is not None:
                    self.local_cache_new.write(f"{redis_code}_REDIS_TYPE={pool_info.get('type', '')}\n")
                    self.local_cache_new.write(f"{redis_code}_REDIS_DATABASE={database}\n")
                    self.local_cache_new.write(f"{redis_code}_REDIS_MODE={mode}\n")
                    self.local_cache_new.write(f"{redis_code}_REDIS_PASSWORD={value}\n")
                    self.local_cache_new.write(f"{redis_code}_REDIS_SENTINEL_PASSWORD={sentinel_value}\n")
                    self.local_cache_new.write(
                        f"{redis_code}_REDIS_MASTER_NAME={pool_info.get('masterName', '')}\n")
                    self.local_cache_new.write(f"{redis_code}_REDIS_POOL_SIZE={pool_info.get('maxIdle', '')}\n")
            except Exception as e:
                logger.warning(f'redis_code: {redis_code}, redis_region: {redis_region} redis_pools: {redis_pools} error: {e}')


    def init_redis(self, redis_config):
        if not redis_config or not redis_config.get('clients'):
            return

        for c in redis_config['clients']:
            pool_name = c['pool']
            database = c.get('database')
            redis_code = c['name']

            pool_info = None
            for p in redis_config['pools']:
                if p['name'] == pool_name:
                    pool_info = p

            if pool_info:
                if pool_info['type'] == 'single':
                    set_env(f'{redis_code}_REDIS_URL', f"{pool_info['host']}:{pool_info['port']}")
                    if self.local_cache is not None:
                        self.local_cache.write(
                            f"{redis_code}_REDIS_URL={pool_info['host']}:{pool_info['port']}\n")
                else:
                    set_env(f'{redis_code}_REDIS_URL', pool_info['nodes'])
                    if self.local_cache is not None:
                        self.local_cache.write(f"{redis_code}_REDIS_URL={pool_info['nodes']}\n")

                value = self.yms_enc_tools.decrypt(pool_info.get('password', ''))
                # sentinel_value = decode_yms_pw(pool_info.get('sentinelPassword', ''))
                sentinel_value = self.yms_enc_tools.decrypt(pool_info.get('sentinelPassword', ''))
                set_env(f'{redis_code}_REDIS_TYPE', f"{pool_info.get('type', '')}")
                set_env(f'{redis_code}_REDIS_DATABASE', f'{database}')
                set_env(f'{redis_code}_REDIS_PASSWORD', value)
                set_env(f'{redis_code}_REDIS_SENTINEL_PASSWORD', sentinel_value)
                set_env(f'{redis_code}_REDIS_MASTER_NAME', f"{pool_info.get('masterName', '')}")
                set_env(f'{redis_code}_REDIS_POOL_SIZE', f"{pool_info.get('max-idle', '')}")

                if self.local_cache is not None:
                    self.local_cache.write(f"{redis_code}_REDIS_TYPE={pool_info.get('type', '')}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_DATABASE={database}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_PASSWORD={value}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_SENTINEL_PASSWORD={sentinel_value}\n")
                    self.local_cache.write(
                        f"{redis_code}_REDIS_MASTER_NAME={pool_info.get('masterName', '')}\n")
                    self.local_cache.write(f"{redis_code}_REDIS_POOL_SIZE={pool_info.get('max-idle', '')}\n")

    def __del__(self):
        self.close_file()
        self.close_file_new()


yms_sys_env = YmsSysEnv()


def request_yms(url, method="GET", params=None, data=None):
    for i in range(1, 5):
        try:
            token = sign_authsdk(url, params, yms_sys_env.ACCESS_KEY, yms_sys_env.ACCESS_SECRET)
            headers = {
                'Content-Type': 'application/json',
                'YYCtoken': token,
                'dcAddr': base64.b64encode(bytes(url, 'utf-8')).decode()
            }
            resp = requests.request(
                method=method, url=url, params=params,
                json=data, verify=False, headers=headers, timeout=5.0
            )
            resp.raise_for_status()
            return resp.json()
        except requests.exceptions.RequestException as e:
            if i == 4:
                raise e
            time.sleep(i * 2)
            try:
                body = e.response.text if hasattr(e.response, "text") else e.response
            except:
                body = ""
            logger.error(
                "request retry time: %s, url: %s, body: %s, error: %s",
                i, url, body, e)

def download_ca_file():
    """
    判断 /etc/ssl/certs/ca-certificates.crt 是否存在
    """
    path = '/etc/ssl/certs/ca-certificates.crt'
    if os.getenv("CA_FILE_URL") is not None and os.path.exists(path):
        logger.info(f"证书{path}已存在，无需下载")
        return
    else:
        logger.info(f"证书{path}不存在")
        return


    # from intelliw.utils.yaml_downloader import download
    # import os
    # workdir = os.getenv('WORKDIR', '/root')
    # ca_dir = os.path.join(workdir, 'cert')
    # os.makedirs(ca_dir, exist_ok=True)
    # ca_url = os.getenv('CA_FILE_URL')
    # if os.getenv('useGpaas') == 'true':
    #     ca_url = f"{os.environ['domain.url']}{ca_url}"
    # if ca_url is None or ca_url == "":
    #     logger.warning("证书下载地址为空")
    #     return
    # try:
    #     # 从URL中提取文件名，或者使用默认文件名
    #     ca_filename = os.path.basename(ca_url) or 'ca.crt'
    #     ca_file_path = os.path.join(ca_dir, ca_filename)
    #     download(ca_url, ca_file_path, True)
    #     logger.info(f"下载证书成功, 证书下载地址: {ca_url}, 保存路径: {ca_file_path}")
    # except Exception as e:
    #     logger.error(f"下载证书失败: {e}, 证书下载地址: {ca_url}")

def read_json_to_dict(file_path, encoding='utf-8'):
    """
    读取 JSON 文件内容，返回以文件名为键的字典
    :param file_path: JSON 文件路径
    :param encoding: 文件编码格式（默认utf-8）
    :return: 格式如 { "filename.json": { ...json content } }
    """
    try:
        with open(file_path, 'r', encoding=encoding) as f:
            data = json.load(f)
        return {os.path.basename(file_path): data}
    except FileNotFoundError:
        print(f"错误：文件 {file_path} 不存在")
        return {}
    except json.JSONDecodeError:
        print(f"错误：文件 {file_path} 不是有效的JSON格式")
        return {}

def run():
    try:
        yms_sys_env.init_config()
        # 1. 使用 AI_CONSOLE_MODULE 获取配置
        succeed_flag = run_new_v2()
        if succeed_flag:
            download_ca_file()
            return

        # 2. 本地配置拉取
        succeed_flag = run_local()
        if succeed_flag:
            download_ca_file()
            return
        
        # 3. 旧版配置拉取
        succeed_flag = run_old()
        if succeed_flag:
            download_ca_file()
            return
        
        logger.error("YMS配置拉取失败，请检查YMS_CONSOLE_ADDRESS是否正确")

    except Exception as e:
        err_msg = traceback.format_exc()
        logger.error("yms配置加载失败: %s, 详细信息: %s", e, err_msg)
    finally:
        try:
            if yms_sys_env.useGpaas:
                from intelliw.config.config import update_by_env   
                logger.info(f"更新config中变量, useGpaas: {yms_sys_env.useGpaas}")
                update_by_env() 
            yms_sys_env.close_file()
            yms_sys_env.close_file_new()
        except:
            pass

def run_local():
    try:
        resp = {}
        # 1. 本地获取配置文件
        if not os.path.exists(yms_sys_env.YMS_ENV_LOCAL_YONBIP_CONFIG_PATH):
            logger.warning(f"配置文件:{yms_sys_env.YMS_ENV_LOCAL_YONBIP_CONFIG_PATH} 不存在，请检查配置文件路径是否正确(或者忽略)")
            return False
        resp.update(read_json_to_dict(yms_sys_env.YMS_ENV_LOCAL_YONBIP_CONFIG_PATH))

        if not os.path.exists(yms_sys_env.YMS_ENV_LOCAL_YMS_MIDDLEWARE_CONFIG_PATH):
            logger.warning(f"配置文件:{yms_sys_env.YMS_ENV_LOCAL_YMS_MIDDLEWARE_CONFIG_PATH} 不存在，请检查配置文件路径是否正确(或者忽略)")
            return False
        resp.update(read_json_to_dict(yms_sys_env.YMS_ENV_LOCAL_YMS_MIDDLEWARE_CONFIG_PATH))

        # 更新环境变量
        yms_sys_env._get_local_cache_new()
        yms_sys_env.init_envs_new(resp.get('yonbip_config.json'))
        # 通过工作坊接口，获取 module_config.json 
        module_config_json_url = f'{os.environ.get("domain.url")}/{yms_sys_env.YMS_CONSOLE_APP_CODE}{yms_sys_env.YMS_CONFIG_MODULE_URL}'
        if yms_sys_env.useGpaas:
            from intelliw.config.config import update_by_env   
            logger.info(f"更新config中变量, useGpaas: {yms_sys_env.useGpaas}")
            update_by_env() 
        status, module_config_resp =  get_module_config(module_config_json_url)
        if status is None or status != 200 or module_config_resp is None or 'SUCCESS' != module_config_resp.get('msg'):
            logger.error(f"通过工作坊接口 {module_config_json_url}，获取 module_config.json 失败, 状态码={status}, resp={module_config_resp}")
            if status is not None and status == 404:
                logger.error(f"退出启动.请检查AI工作坊(iuap-ai-console-service)是否启动, 如果没有启动, 请先启动AI工作坊, 再重启本服务.")
                exit(-1)
            return False
        else:
            logger.info("通过工作坊接口 %s，获取 module_config.json 成功, 状态码=%s", module_config_json_url, status)
        
        resp.update(module_config_resp.get('data', {}))

        yms_sys_env.init_envs_new(resp.get('module_config.json'))
        filtered_clients = yms_sys_env.flat_clients()
        try:
            yms_sys_env.init_redis_new(resp.get('yms_middleware.json'), filtered_clients.get('redis'))
            yms_sys_env.init_connection_pools_new(resp.get('yms_middleware.json'), {f'{yms_sys_env.YMS_CONSOLE_APP_CODE}_datasource'})
            yms_sys_env.init_connection_pools_new(resp.get('yms_middleware.json'), {f'{yms_sys_env.YMS_ALGVEC_APP_CODE}_dataSource'})
        except Exception as e:
            logger.error("yms配置本地加载失败: %s", e)

        logger.info("执行yms配置本地加载成功")
        # 写入文件
        if "DEBUG" == os.environ.get(logger_code):
            with open(yms_sys_env.YMS_ENV_RESP_CONFIG_NEW_FILE, 'w') as f:
                f.write(json.dumps(resp, indent=4))
        return True


    except Exception as e:
        err_msg = traceback.format_exc()
        logger.error("yms配置本地加载失败: %s, 详细信息: %s", e, err_msg)
        return False
    finally:
        try:
            yms_sys_env.close_file()
            yms_sys_env.close_file_new()
        except:
            pass

def run_new_v2():
    '''
    通过工作坊接口获取全量的配置文件(替换 run_new)
    会获取不到 iuap-aip-vpa 配置
    '''
    if not yms_sys_env.AI_CONSOLE_MODULE:
        logger.warning("未配置 AI_CONSOLE_MODULE，不通过工作坊接口拉取YMS配置")
        return False

    try:
        params = {
                    'key': 'ALL'
                }
        status, all_config_resp = get_module_config(yms_sys_env.AI_CONSOLE_MODULE,
                           params= params
                           )
        resp = {}

        if status is None or status != 200 or all_config_resp is None or 'SUCCESS' != all_config_resp.get('msg'):
            logger.error(f"通过工作坊接口 {yms_sys_env.AI_CONSOLE_MODULE}，获取YMS配置失败, resp={all_config_resp}")
            if status is not None and status == 404:
                logger.error(f"退出启动.请检查AI工作坊(iuap-ai-console-service)是否启动, 如果没有启动, 请先启动AI工作坊, 再重启本服务.")
                exit(-1)
            return False
        else:
            logger.info(f"通过工作坊接口 {yms_sys_env.AI_CONSOLE_MODULE}，获取YMS配置成功")
        resp = all_config_resp.get('data')

        if not resp or not isinstance(resp, dict):
            logger.error(f"通过工作坊接口 {yms_sys_env.AI_CONSOLE_MODULE}，获取YMS配置失败, resp={all_config_resp}")
            return False
        
        yms_sys_env._get_local_cache_new()
        yms_sys_env.init_envs_new(resp.get('yonbip_config.json'))
        yms_sys_env.init_envs_new(resp.get('module_config.json'))
        filtered_clients = yms_sys_env.flat_clients()
        try:
            yms_sys_env.init_redis_new(resp.get('yms_middleware.json'), filtered_clients.get('redis'))
            yms_sys_env.init_connection_pools_new(resp.get('yms_middleware.json'), {f'{yms_sys_env.YMS_CONSOLE_APP_CODE}_datasource'})
            yms_sys_env.init_connection_pools_new(resp.get('yms_middleware.json'), {f'{yms_sys_env.YMS_ALGVEC_APP_CODE}_dataSource'})
        except Exception as e:
            logger.error("通过工作坊接口 {yms_sys_env.AI_CONSOLE_MODULE},YMS配置下载失败: %s", e)
            return False
        logger.info(f"通过工作坊接口 {yms_sys_env.AI_CONSOLE_MODULE}, 更新YMS配置成功")
        if "DEBUG" == os.environ.get(logger_code):
            with open(yms_sys_env.YMS_ENV_RESP_CONFIG_NEW_FILE, 'w') as f:
                f.write(json.dumps(resp, indent=4))
        return True


    except Exception as e:
        err_msg = traceback.format_exc()
        logger.error("yms配置下载失败: %s, 详细信息: %s", e, err_msg)
        return False
    finally:
        try:
            yms_sys_env.close_file()
            yms_sys_env.close_file_new()
        except:
            pass

def run_new():
    '''
    通过yms v2 接口获取配置文件(使用 run_new_v2后， 本接口弃用, 暂时保留)
    '''
    try:
        params = {
                    'apps': [f'{yms_sys_env.YMS_CONSOLE_APP_CODE}',"iuap-aip-vpa"],
                    'env': yms_sys_env.YMS_CONSOLE_ACTIVE,
                    "version": "2.0.0", 
                    "keyStore": "true",
                    "c_isolate":"default"
                }
        resp = request_yms(yms_sys_env.YMS_CONFIG_NEW_FILE_ADDRESS,
                           params= params
                           )
        if resp.get('success') == 'false':
            logger.error(f"{resp['error_code']}:{resp['error_message']}")
            return False
        if "DEBUG" == os.environ.get(logger_code):
            with open(yms_sys_env.YMS_ENV_RESP_CONFIG_NEW_FILE, 'w') as f:
                f.write(json.dumps(resp, indent=4))
        
        yms_sys_env._get_local_cache_new()
        yms_sys_env.init_envs_new(resp.get('yonbip_config.json'))
        yms_sys_env.init_envs_new(resp.get('module_config.json'))
        filtered_clients = yms_sys_env.flat_clients()
        yms_sys_env.init_redis_new(resp.get('yms_middleware.json'), filtered_clients.get('redis'))
        yms_sys_env.init_connection_pools_new(resp.get('yms_middleware.json'), {f'{yms_sys_env.YMS_CONSOLE_APP_CODE}_datasource'})
        yms_sys_env.init_connection_pools_new(resp.get('yms_middleware.json'), {f'{yms_sys_env.YMS_ALGVEC_APP_CODE}_dataSource'})
        logger.info("执行新的YMS配置拉取成功")
        return True


    except Exception as e:
        err_msg = traceback.format_exc()
        logger.error("yms配置下载失败: %s, 详细信息: %s", e, err_msg)
        return False
    finally:
        try:
            yms_sys_env.close_file()
            yms_sys_env.close_file_new()
        except:
            pass

def run_old():
    if not yms_sys_env.YMS_CONSOLE_ADDRESS:
        logger.warning("未配置YMS_CONSOLE_ADDRESS，不进行YMS配置拉取")
        return False
    try:
        resp = request_yms(yms_sys_env.YMS_CONFIG_FILE_ADDRESS,
                           params={
                               'app': yms_sys_env.YMS_CONSOLE_APP_CODE,
                               'env': yms_sys_env.YMS_CONSOLE_ACTIVE,
                               "version": 1, 
                               "datacenter": "",
                               "keyStore": "",
                               }
                           )
        if resp.get('success') == 'false':
            logger.error(f"{resp['error_code']}:{resp['error_message']}")
            return False
        if "DEBUG" == os.environ.get(logger_code):
            with open(yms_sys_env.YMS_ENV_RESP_CONFIG_FILE, 'w') as f:
                f.write(json.dumps(resp, indent=4))
        yms_sys_env.init_envs(resp.get('environment'))
        yms_sys_env.init_redis(resp.get('redis'))
        logger.info(f"通过 {yms_sys_env.YMS_CONFIG_FILE_ADDRESS} 执行YMS配置拉取成功")
        return True

    except Exception as e:
        err_msg = traceback.format_exc()
        logger.error("yms配置下载失败: %s, 详细信息: %s", e, err_msg)
        return False
    finally:
        try:
            yms_sys_env.close_file()
            yms_sys_env.close_file_new()
        except:
            pass



#!/usr/bin/env python
# coding: utf-8

import inspect
import json
import time
import errno
import shutil
import os
import cProfile
import zipfile
import traceback
from functools import wraps
from intelliw.utils import message
from intelliw.utils.exception import CheckpointException, SnapshotException
from intelliw.utils.storage_service import FileTransferDevice
from intelliw.utils.logger import _get_framework_logger, Logger
from intelliw.config import config
from intelliw.utils import generate_random_str, get_json_encoder

logger = _get_framework_logger()

save_index = 0
curkey_r = generate_random_str(32)


def zipdir(model_path):
    outpath = '/tmp/model.zip'
    with zipfile.ZipFile(outpath, 'w', zipfile.ZIP_DEFLATED) as zipf:
        if os.path.isdir(model_path):
            for root, _, files in os.walk(model_path):
                relative_path = root.replace(model_path, "")
                for file in files:
                    logger.info("压缩文件 {}".format(os.path.join(root, file)))
                    zipf.write(os.path.join(root, file),
                               os.path.join(relative_path, file))
        elif os.path.isfile(model_path):
            zipf.write(model_path, os.path.basename(model_path))
    return outpath


def decorator_report_train_info(function, reporter=None):
    from intelliw.utils import get_json_encoder

    @wraps(function)
    def wrapper(loss, lr, iter, batchsize, **kwargs):
        if reporter is not None:
            info = json.dumps({
                "loss": loss,
                "lr": lr,
                "iter": iter,
                "batchsize": batchsize,
                "timestamp": int(time.time() * 1000),
                "other": kwargs
            }, cls=get_json_encoder())
            reporter.report(message.CommonResponse(
                200, 'report_train_info', '', str(info)))
        return function(loss, lr, iter, batchsize, **kwargs)
    return wrapper


def decorator_report_val_info(function, reporter=None):
    from intelliw.utils import get_json_encoder

    @wraps(function)
    def wrapper(*args, **kwargs):
        if reporter is not None:
            val = {}

            # process args
            varnames = function.__code__.co_varnames
            offset = 0
            for i in range(len(varnames)):
                if varnames[i] == 'self' or varnames[i] == 'args' or varnames[i] == 'kwargs':
                    offset = offset - 1
                    continue
                if i + offset < len(args):
                    val[varnames[i]] = args[i + offset]
                else:
                    val[varnames[i]] = None

            # process kwargs
            for k, v in kwargs.items():
                val[k] = v

            data = {
                "modelInstanceId": config.INSTANCE_ID,
                "tenantId": config.TENANT_ID,
                "valuationResult": val
            }
            reporter.report(message.CommonResponse(200, 'report_val_info', '', json.dumps(
                data, cls=get_json_encoder(), ensure_ascii=False)))
        return function(*args, **kwargs)
    return wrapper

def decorator_report_train_extra_info(function, reporter=None):
    from intelliw.utils import get_json_encoder

    @wraps(function)
    def wrapper(*args, **kwargs):
        if reporter is not None:
            val = {}

            # process args
            varnames = function.__code__.co_varnames
            offset = 0
            for i in range(len(varnames)):
                if varnames[i] == 'self' or varnames[i] == 'args' or varnames[i] == 'kwargs':
                    offset = offset - 1
                    continue
                if i + offset < len(args):
                    val[varnames[i]] = args[i + offset]
                else:
                    val[varnames[i]] = None

            # process kwargs
            for k, v in kwargs.items():
                val[k] = v

            reporter.report(message.CommonResponse(200, 'report_train_extra_info', '', json.dumps(
                val, cls=get_json_encoder(), ensure_ascii=False)))
        return function(*args, **kwargs)
    return wrapper

# decorator_save 存储模型文件到云存储
def decorator_save(function, reporter=None):

    @wraps(function)
    def wrapper(*args, **kwargs):
        # 分布式训练 slave不需要保存模型
        if config.FRAMEWORK_MODE == config.FrameworkMode.DistTrain and \
                not config.DIST_IS_MASTER:
            logger.info("分布式训练slave服务不需要保存模型文件")
            return None

        # 如果用户输入的是绝对路径，就使用输入的路径
        user_path = args[0]
        if os.path.isabs(user_path):
            model_path = user_path
        else:
            hpath = os.path.join('/tmp', generate_random_str(16))
            os.makedirs(hpath)
            model_path = os.path.join(hpath, user_path)

        # 创建模型保存文件夹
        if not os.path.exists(model_path):
            logger.info("目录不存在， 自动创建 {}".format(model_path))
            try:
                os.makedirs(model_path)
            except OSError as e:
                if e.errno == errno.EEXIST and os.path.isdir(model_path):
                    pass
                else:
                    logger.error("保存模型错误:  创建目录失败")
                    reporter.report(str(message.CommonResponse(500, "train_save",
                                                               "保存模型错误:  创建目录失败 {}".format(model_path))))

        # 设置save_index
        global save_index
        is_checkpoint = False
        is_snapshot = False
        if function.__name__ == 'save_checkpoint':
            is_checkpoint = True
            if kwargs.get("save_best_only") is True:
                save_index = "best"
            elif "max_to_keep" in kwargs:
                # save_index类型为str类型，说明之前执行过save()方法或执行过save_checkpoint()方法save_best_only为true的情况，不符合逻辑
                if isinstance(save_index, str):
                    raise CheckpointException()
                max_to_keep = kwargs.get("max_to_keep")
                if max_to_keep < 0:
                    save_index = (save_index + 1) % config.CHECKPOINT_SAVE_MAX
                elif max_to_keep > 0:
                    save_index = (
                        save_index + 1) % max_to_keep % config.CHECKPOINT_SAVE_MAX
        elif function.__name__ == 'save_snapshot':
            is_snapshot = True
            version = kwargs.get('version')
            if version is None or not isinstance(version, int):
                raise SnapshotException()
            save_index = version

        else:
            save_index = "model"
        # 执行用户的保存函数
        result = function(model_path)

        # 上传模型
        if reporter is not None:
            _, outpath = __push_model_to_cloud(
                reporter, model_path, save_index, is_checkpoint, is_snapshot, kwargs)
            try:
                os.remove(outpath)
                shutil.rmtree(hpath, ignore_errors=True)
            except:
                pass
        else:
            logger.info("保存模型错误: reporter is  None")

        return result
    return wrapper


def __push_model_to_cloud(reporter, model_path, index, is_checkpoint, is_snapshot, kwargs):
    from intelliw.utils.storage_service import StorageService

    try:
        out_path = zipdir(os.path.abspath(model_path))
        save_fn = config.INSTANCE_ID or curkey_r
        curkey = os.path.join(config.STORAGE_SERVICE_PATH, save_fn, str(index))
        uploader = StorageService(curkey, "upload")
        logger.info(
            f"模型上传地址{uploader.service_url}, checkpoint模式：{is_checkpoint}, snapshot模式：{is_snapshot}")
        try:
            uploader.upload(out_path)

            logger.info(
                f"上传模型文件成功：{curkey}, checkpoint模式：{is_checkpoint}, snapshot模式：{is_snapshot}")
            if is_checkpoint:
                data = {'modelPath': curkey, 'epoch': kwargs.get(
                    'epoch'), 'indice': str(kwargs.get('indice'))}
                request = json.dumps(
                    data, cls=get_json_encoder(), ensure_ascii=False)
                business_type = "save_checkpoint"
            elif is_snapshot:
                data = {'modelPath': curkey, 'epoch': kwargs.get(
                    'version'), 'indice': str(kwargs.get('indice'))}
                request = json.dumps(
                    data, cls=get_json_encoder(), ensure_ascii=False)
                business_type = "save_snapshot"
            else:
                request = [curkey]
                business_type = "save_model"

            reporter.report(message.CommonResponse(
                200, 'train_save', 'success', request, businessType=business_type))
        except Exception:
            err_info = traceback.format_exc()
            logger.info(
                f"上传模型文件失败: {err_info}, checkpoint模式：{is_checkpoint}, snapshot模式：{is_snapshot}")
            reporter.report(str(message.CommonResponse(
                500, "train_save", f"保存模型错误 {err_info}")))
    except Exception as e:
        stack_info = traceback.format_exc()
        reporter.report(str(message.CommonResponse(500, "train_save",
                                                   "保存模型错误 {}, stack: \n {}".format(e, stack_info))))
    return curkey, out_path


def flame_prof_process(function):
    @wraps(function)
    def wrapper(*args, **kwargs):
        if not config.FLAME_PROF_MODE:
            return function(*args, **kwargs)
        else:
            prof_file = "./request.prof"

            pr = cProfile.Profile()
            pr.enable()
            result = function(*args, **kwargs)
            pr.disable()
            pr.dump_stats(prof_file)
            FileTransferDevice(prof_file, "flame_prof")
            logger.info("\033[33mFlame Performance Complete: Save file to ./request.prof\033[0m")
            return result
    return wrapper


def make_decorators_server(instance, reporter=None):
    # report_train_info
    if (hasattr(instance, 'report_train_info')) and inspect.ismethod(instance.report_train_info):
        instance.report_train_info = decorator_report_train_info(
            instance.report_train_info, reporter)

    # report_train_extra_info
    if (hasattr(instance, 'report_train_extra_info')) and inspect.ismethod(instance.report_train_extra_info):
        instance.report_train_extra_info = decorator_report_train_extra_info(
            instance.report_train_extra_info, reporter)

    # report_val_info
    if (hasattr(instance, 'report_val_info')) and inspect.ismethod(instance.report_val_info):
        instance.report_val_info = decorator_report_val_info(
            instance.report_val_info, reporter)

    # save model
    if (hasattr(instance, 'save')) and inspect.ismethod(instance.save):
        instance.save = decorator_save(instance.save, reporter)

    # save model
    if (hasattr(instance, 'save_checkpoint')) and inspect.ismethod(instance.save_checkpoint):
        instance.save_checkpoint = decorator_save(
            instance.save_checkpoint, reporter)
    
    # save snapshot
    if (hasattr(instance, 'save_snapshot')) and inspect.ismethod(instance.save_snapshot):
        instance.save_snapshot = decorator_save(
            instance.save_snapshot, reporter)


def make_decorators_local(instance):
    if (hasattr(instance, 'get_user_logger')) and inspect.isfunction(instance.get_user_logger):
        instance.get_user_logger = Logger()._get_logger



'''
Author: hexu
Date: 2021-10-14 14:54:05
LastEditTime: 2023-05-26 19:54:39
LastEditors: Hexu
Description: 从语义模型获取模型数据集
FilePath: /iw-algo-fx/intelliw/datasets/datasource_semantic.py
'''
from concurrent.futures import ThreadPoolExecutor
import os
import json
from intelliw.datasets.datasource_base import AbstractDataSource, DataSourceReaderException, \
    DataSourceType, DatasetSelector
from intelliw.utils import iuap_request
from intelliw.config import config
from intelliw.utils.logger import _get_framework_logger
from intelliw.config import config

logger = _get_framework_logger()


class DataSourceSemanticData(AbstractDataSource):
    """
    语义模型数据源
    """

    def __init__(self, data_address, table_id, user_id, fields=None, data_filter_condition = None):
        """
        语义模型数据源
        :param data_address:   获取数据 url
        :param fields: 获取数据表字段
        :param table_id:   表Id
        """
        self.data_address = data_address
        self.table_id = table_id
        self.user_id = user_id
        self.fields = fields
        self.__total = None
        self.__mata = []
        self.conditions = None
        if data_filter_condition is None or len(data_filter_condition) == 0:
            data_filter_condition = config.DATA_FILTER_CONDITION
            try:
                self.conditions = json.loads(data_filter_condition).get('conditions')
                logger.info("parse data_filter_condition success, config is %s", data_filter_condition)
            except Exception as e:
                self.conditions = None

    def total(self):
        """获取数据总行数"""
        if self.__total is not None:
            return self.__total

        data = {
            'entityId': self.table_id,
            'pager': {
                "pageIndex": 1,
                "pageSize": 1
            }
        }
        if self.conditions is not None:
            data['conditions'] = self.conditions
        headers = {'user_id': self.user_id}
        response = iuap_request.post_json(
            self.data_address, json=data, headers=headers, timeout=config.DATA_SOURCE_READ_TIMEOUT, auth_type=iuap_request.AuthType.YHT)
        if response.status != 200:
            msg = f"获取行数失败，url: {self.data_address}, response: {response}"
            raise DataSourceReaderException(msg)
        row_data = response.json

        self.__total = 0
        try:
            count = row_data["data"]["pager"]['recordCount']
            if isinstance(count, int):
                self.__total = count
            
            meta = row_data["data"]["meta"]
            if isinstance(meta, list):
                self.__meta = meta
        except Exception as e:
            msg = f"获取行数返回结果错误, response: {row_data}, request_url: {self.data_address}"
            raise DataSourceReaderException(msg)
        return self.__total
    
    def get_meta(self):
        return self.__meta
    def reader(self, pagesize=10000, offset=0, limit=0, transform_function=None, dataset_type='train_set'):
        return self.__Reader(self.data_address, self.table_id, self.fields, self.total(), self.get_meta(), self.user_id, pagesize,
                             transform_function, self.conditions)

    class __Reader:
        def __init__(self, data_address, table_id, fields, total, meta, user_id, pagesize=5000, transform_function=None, conditions: list = None):
            """
            eg. 91 elements, page_size = 20, 5 pages as below:
            [0,19][20,39][40,59][60,79][80,90]
            offset 15, limit 30:
            [15,19][20,39][40,44]
            offset 10 limit 5:
            [10,14]
            """
            self.logger = _get_framework_logger()
            self.data_address = data_address
            self.table_id = table_id
            self.fields = fields
            self.origin_page_size = pagesize
            self.page_size = max(100, pagesize)
            self.page_num = 1
            self.total_read = 0
            self.total_rows = total
            self.user_id = user_id
            self.meta = meta
            self.worker = min(3, max(1, os.cpu_count()))
            self.transform_function = transform_function
            self.conditions = conditions
            logger.info(f'数据获取中: 启用的获取线程 {self.worker} 条')

        def get_data_bar(self):
            """数据拉取进度条"""
            try:
                proportion = round(
                    (self.total_read / self.total_rows) * 100, 2)
                logger.info(
                    f"数据获取中: {self.total_read}/{self.total_rows}, 进度{proportion}%")
            except:
                pass

        @property
        def iterable(self):
            return True

        def __iter__(self):
            return self

        def reset(self, fields: list=None, page_size: int=None):
            """ 迭代重置 """
            self.page_num = 1
            self.total_read = 0
            self.fields = fields
            if page_size:
                self.page_size = page_size
            else:
                self.page_size = self.origin_page_size
            logger.info(f'数据读取重置, page_num: {self.page_num}, total_read: {self.total_read}, fields: {self.fields}, page_size: {self.page_size}') 

        def get_meta(self):
            """获取元数据"""
            return self.meta
        
        def get_total_rows(self):
            """获取总数据行数"""
            return self.total_rows

        

        def __next__(self):
            if self.total_read >= self.total_rows:
                logger.info('数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                raise StopIteration

            self.get_data_bar()

            try:
                with ThreadPoolExecutor(max_workers=self.worker) as executor:
                    futures, data_result = [], []
                    for i in range(self.worker):
                        futures.append(executor.submit(
                            self._read_page, self.page_num, self.page_size, self.conditions
                        ))
                        self.page_num += 1

                    for f in futures:
                        data_result.extend(f.result())
                    self.total_read += len(data_result)
                    if len(data_result) == 0:
                        logger.info(
                            '数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                        raise StopIteration

                    return_data = {"result": data_result, "meta": self.meta}
                    if self.transform_function is not None:
                        return_data = self.transform_function(return_data)
                    return return_data
            except Exception as e:
                logger.exception(
                    f"语义模型数据源读取失败, data_address: [{self.table_id}]")
                raise DataSourceReaderException(f'语义模型数据源读取失败: {e}')

        def _get_meta(self, meta):
            if self.meta == []:
                self.meta = meta
            return self.meta

        def _data_process(self, result):
            assert result is not None, "语义模型数据接口未返回任何有效数据"
            data = result["data"]
            assert data is not None, f"接口返回无数据, 请联系数据负责人：{result}"
            if len(data) > 0:
                self._get_meta(data['meta'])
            return data['result']

        def _read_page(self, page_index, page_size, conditions: list = None):
            """
            语义模型获取数据接口，分页读取数据
            :param page_index: 页码，从 0 开始
            :param page_size:  每页大小

            此接口：pageIndex 代表起始下标（不是页）， pagesize代表每页数据的数量， pagecount代表获取几页
                   但是返回的数据类型是[{},{}] 而不是 [[{},{}],[]], 所以保证pageSize和pageCount中某一个数为1的时候， 另一个参数就可以当size使用（很迷惑）
            例如： {'id': self.table_id, 'pageIndex': 1,'pageSize': 10, 'pageCount': 1} 和 {'id': self.table_id, 'pageIndex': 1,'pageSize': 1, 'pageCount': 10} 的结果完全一致
            :return:
            """
            request_data = {
                'entityId': self.table_id,
                'pager': {
                    "pageIndex": page_index,
                    "pageSize": page_size
                }
            }
            if conditions is not None:
                request_data['conditions'] = conditions
            
            if self.fields is not None:
                request_data['fields'] = [ {"name": field} for field in self.fields ]

            headers = {'user_id': self.user_id}
            response = iuap_request.post_json(
                url=self.data_address, json=request_data, headers=headers, timeout=config.DATA_SOURCE_READ_TIMEOUT,
                auth_type=iuap_request.AuthType.YHT)
            response.raise_for_status()
            result = response.json
            return self._data_process(result)


DatasetSelector.register_func(DataSourceType.SEMANTIC, DataSourceSemanticData, {
    "data_address": "INPUT_ADDR",
    "table_id": "INPUT_MODEL_ID",
    "user_id": "USER_ID",
    })



class Algorithm:
    """
    算法类， 导入的算法必须定义class Algorithm， 并放入 algorithm.py 文件中
    """

    def __init__(self, parameters):
        """
        初始化， parameters 为 dict格式的参数信息， 比如
        {
            "params1": "value1",
            "params1", 2
        }
        model_manager_mode 为 True 时, 会使用模型管理器加载模型, 否则走正常的 load 函数
        """
        self.parameters = parameters
        self.logger = self.parameters['framework_log']  # 日志

        self.model_manager_mode = False
        self.model_manager = None
        pass

    def load_with_manager(self, path):
        '''
        使用模型管理器加载模型, 需要返回模型实例
        :param path:
        :return:
        '''
        model = None
        pass
        return model
    def load(self, path):
        """
        加载模型
            不需要调用，只需要在这里实现加载模型的方法，并赋值给一个属性就可以，例如：
            self.model = xxxx.load(path)
            上线流程：算法框架会将模型文件放在model/下，然后进行函数的执行
            本地流程：可以通过model.yaml中location字段进行配置，然后会传入path参数
            Args:
                path : 模型文件路径，根据 model.yaml 文件中的 location 字段指定
            Returns:
                无
        """
        pass

    def save(self, path):
        """
        保存模型， 保存为 path 的文件或文件夹会被框架程序自动打包上传到服务器
        如果是path是文件夹，需要加上/， 系统会创建该文件夹
            Args:
                path : 模型文件路径， 文件名或文件夹名，根据算法自定义
            Returns:
                无
        """
        pass

    def train(self, **kwargs):
        """
        模型训练
            Args:
                self.train_set : 训练数据
                self.valid_set : 验证数据
                self.test_set : 测试数据

                content: [{
                    "meta": [   // 表头
                        {"code": "column_a"},
                        {"code": "column_b"}
                    ],
                    "result": [ // 表体
                        ["line_1_column_1", "line_1_column_2"],
                        ["line_2_column_1", "line_2_column_2"]
                    ]
                },...]

                kwargs: 特征工程后返回的数据
                {
                    "target_cols": [{"col": 目标列下标,"column_name": 目标列名称},...] // 目标列
                    "category_cols": [1,2,3]     // 类别列下标
                    "column_meta": [Employee_ID,Gender,Age,...]    // 原始数据的列名
                    "column_relation_df": Dataframe   // the map between the input columns and output columns, if column is droped, it will not exist in column_relation_df!
                }
            Returns:
                无
        """
        pass

    async def infer(self, infer_data):
        """
        推理
            推理服务Args:
                infer_data : 推理请求数据, json输入的参数, 类型可以为列表/字符串/字典

                如果使用特征工程, infer_data格式有硬性要求:
                {
                    "data": 任意格式，如果配置特征工程，此数据为处理后的数据 # 推理数据
                    "original_data":  原始输入的data ，防止数据在通过特征处理后丢失原先的数据
                }

                self.request dict[str, object]: 请求体
                    |- self.request.header  object: 请求头
                    |- self.request.files    ImmutableMultiDict[str, FileStorage]: 文件列表, 可以参照flask的文件读取
                    |- self.request.query   Dict[str]str: url参数
                    |- self.request.form    Dict[str]list: form表单参数
                    |- self.request.json    object: json参数
                    |- self.request.body    bytes: raw request data
                    批处理专有参数
                    |- self.request.batch_params List[Dict] 批处理输入参数,为请求中的json数据,用作任务间参数传递,请直接修改参数,不要deepcopy

            批处理服务Args:
                infer_data : 批处理输入数据
                            [{
                                "meta": [   // 表头
                                    {"code": "column_a"},
                                    {"code": "column_b"}
                                ],
                                "result": [ // 表体
                                    ["line_1_column_1", "line_1_column_2"],
                                    ["line_2_column_1", "line_2_column_2"]
                                ]
                            }]
                self.request dict[str, object]: 请求体
                    |- self.request.batch_params List[Dict]  批处理接口(/batch-predict)输入参数,为请求中的json数据,用作任务间参数传递,请直接修改参数,不要deepcopy

            Returns:
                推理结果, json
        """
        pass

    def report_train_info(self, loss=0, lr=1, iter=0, batchsize=1, **kwargs):
        """
        上报训练信息
            Args:
                loss： 损失函数loss值
                lr： 学习率
                iter： 当前iteration
                batchsize： 每批次数据量大小
                kwargs:  额外上报内容
            Returns:
                无
        """
        pass

    def report_val_info(self, **kwargs):
        """
        上报验证信息
            Args:
                无
            Returns:
                无
        """
        pass



import numpy as np

KEY_AXIS = "axis"
KEY_CHAR_SET = "char_set"


def argmax(data, params):
    if type(data) is not np.ndarray:
        data = np.array(data)
    if KEY_AXIS not in params:
        raise Exception("expected param axis in post func argmax")
    axis = params[KEY_AXIS]
    return np.argmax(data, axis=axis).reshape(data.shape[1])


def post_process(data, params):
    if KEY_CHAR_SET not in params:
        raise Exception("expected param char_set in post func post_process")
    char_set = params[KEY_CHAR_SET]
    res = {'data': ""}
    for index in data:
        res['data'] += char_set[index]
    res['status'] = 1
    return res



import numpy as np

KEY_FACTOR = "factor"
KEY_DIMENSION = "dimension"
KEY_SHAPE = "shape"
KEY_AXIS = "axis"


def normalize(data, params):
    if type(data) is not np.ndarray:
        raise Exception(
            "expected input data type numpy.ndarray in func normalize")
    value = params[KEY_FACTOR]
    return data / value


def reshape(data, params):
    if type(data) is not np.ndarray:
        data = np.array(data)
    if KEY_SHAPE not in params:
        raise Exception("expected param shape in matrix func reshape")
    shape = params[KEY_SHAPE]

    if len(data.shape) > len(shape):
        raise Exception("expected {} dimensions, but got array with shape {}".format(
            len(shape), data.shape))
    if len(data.shape) < len(shape):
        axis = []
        for index in range(len(shape)):
            if shape[index] == 1:
                axis.append(index)
        data = np.expand_dims(data, axis=axis)

    return np.reshape(data, shape)


def transpose(data, params):
    if type(data) is not np.ndarray:
        data = np.array(data)
    if KEY_SHAPE not in params:
        raise Exception("expected param shape in matrix func transpose")
    axes = params[KEY_SHAPE]
    if len(data.shape) != len(axes):
        raise Exception("expected {} dimensions, but got array with shape {}".format(
            len(axes), data.shape))
    return np.transpose(data, axes)


def argmax(data, params):
    axis = params[KEY_AXIS]
    return np.argmax(data, axis=axis)



from functools import wraps
import threading
from typing import Union, List
from intelliw.utils import iuap_request


try:
    from eurekapy import http_client, eureka_client
    has_eurekapy_package = True

    # AuthHttpClient 重写HttpClient， 方便做加签
    class AuthHttpClient(http_client.HttpClient):
        def urlopen(self, request: Union[str, http_client.HttpRequest] = None,
                    data: bytes = None, timeout: float = None) -> http_client.HttpResponse:

            if isinstance(request, http_client.HttpRequest):
                req = request
            elif isinstance(request, str):
                req = http_client.HttpRequest(request)
            else:
                raise http_client.URLError("Unvalid URL")
            req.add_header("YYCtoken", iuap_request.sign_authsdk(req.url, None))
            req.add_header("Content-Type", "application/json")
            # import requests
            # resp = requests.request(
            #     req.method if req.method else "get", req.url, headers=req.headers, data=data)
            # print("-------", resp.text, resp.status_code)
            return super().urlopen(req, data, timeout)

    http_client.set_http_client(AuthHttpClient())
except ImportError:
    has_eurekapy_package = False


class LinkServerError(Exception):
    pass


class LinkServiceType:
    RPC = 'rpc'
    HTTP = 'http'
    LOCAL = 'local'


class LinkBase:
    def __init__(self) -> None:
        self.config = None

    def __call__(self, config: List[dict] = None):
        def wrapper(function):
            @wraps(function)
            def inner(*args, **kwargs):
                return function(*args, **kwargs)
            return inner
        return wrapper


class LinkService:
    """LinkService提供模型编排功能

    客户端生成:
      1 不注册服务： 通过open()方法进行客户端连接, 但是不会将此服务注册在服务中心
      2 注册服务: 通过register()方法进行客户端连接, 并将此服务注册在注册中心

    模型编排:
      1 通过设置self.config, 进行自动编排
        config格式如下:
            config = [
                {"name": "test1", "method": "get", "router": "/api-get", "type": "rpc"},
                {"name": "test2", "method": "get", "router": "http://ip:port/api-get", "type": "http"},
            ]

      2 手动调用rpc服务
        创建客户端连接后， 通过do_server()进行远程服务调用

    """
    _instance_lock = threading.Lock()

    def __new__(cls):
        if not hasattr(LinkService, "_instance"):
            with LinkService._instance_lock:
                if not hasattr(LinkService, "_instance"):
                    LinkService._instance = object.__new__(cls)
        return LinkService._instance

    def __init__(self) -> None:
        self._client = eureka_client
        self.config = None

    def client(self, eureka_server: str, provider_id: str, should_register: bool = False, server_name: str = "", server_iports: list = [], profile: str = "test"):
        """生成客户端连接

        Args:
            eureka_server (str): eureka服务地址
            providerId (str): 租户id.
            should_register (bool, optional): 是否注册当前服务. Defaults to False.
            server_iports (int, optional): 注册服务ip和端口, should_register is True 是必填. Defaults to [],  e.g.:["192.168.0.1:8080", "192.168.0.2:8080"]
            server_name (str, optional): 注册服务名称, should_register is True 是必填. Defaults to 0.
            profile (str, optional): 服务命名空间
        """
        if should_register:
            self.register(eureka_server, provider_id,
                          server_name, server_iports, profile)
        else:
            self.open(eureka_server, provider_id, profile)

    def open(self, eureka_server: str, provider_id: str, profile: str = "test"):
        """生成客户端连接, 但是不会将此服务注册在服务中心

        Args:
            eureka_server (str): eureka服务地址.
            providerId (str): 租户id.
            profile (str, optional): 服务命名空间
        """
        self._client.init(
            eureka_server=eureka_server,
            should_register=False,
            profile=profile,
            prefer_same_zone=False,
            prefer_same_profile=True,
            payload={"providerId": provider_id}
        )

    def register(self, eureka_server: str, provider_id: str, server_name: str, server_iports: list = [], profile: str = "test"):
        """生成客户端连接, 并将此服务注册在注册中心

        Args:
            eureka_server (str): eureka服务地址.
            provider_id (str): 租户id.
            server_name (str): 注册服务名称
            server_iports (list, optional): 注册服务端口
            profile (str, optional): 服务命名空间
        """
        if not server_iports or not server_name or not eureka_server or not provider_id:
            raise LinkServerError(
                f"register must need:\nip and port:{server_iports}\nname: {server_name}\neureka_server: {eureka_server}\nprovider_id: {provider_id}")

        # 会根据ip数量开启多组心跳检测
        for iport in server_iports:
            ip, port = iport.split(":")
            instance_id = f'{iport}@{provider_id}'
            metadata = {
                "instanceId": instance_id,
                "context": "",
                "all_iports": f"[\"{iport}\"]",
            }
            self._client.init(
                app_name=server_name,
                instance_ip=ip,
                instance_port=int(port),
                instance_id=instance_id,
                metadata=metadata,
                eureka_server=eureka_server,
                ha_strategy=eureka_client.HA_STRATEGY_RANDOM,
                payload={"providerId": provider_id, "appRemark": ""},
                profile=profile,
                renewal_interval_in_secs=10,
            )

    def do_server(self, name: str, router: str, data: Union[dict, str, bool, int] = None, method: str = "POST"):
        """手动请求rpc服务

        Args:
            name (str): 注册服务名称
            router (str): 请求路由 e.g./predict
            data (Union[dict, str, bool, int], optional): 请求内容. Defaults to None.
            method (str, optional): 请求方式. Defaults to "GET".

        Returns:
            Union[dict, str, bool, int], optional:  response.json
        """
        try:
            data = self._rpc_request(name, router, method, data)
        except Exception as e:
            raise LinkServerError(
                f"link server error:\n server info: {name}-{router}-{method}, error msg: {e}")
        return data

    def __call__(self, config: List[dict] = None):
        """调用装饰器, 自动编排使用

        Args:
            config (List[dict], optional): 调用配置. Defaults to None. 优先使用传入配置
        """
        def wrapper(function):
            @wraps(function)
            def inner(*args, **kwargs):
                # 无调用
                if not config and not self.config:
                    return function(*args, **kwargs)

                # 优先传入的配置
                self.config = config or self.config
                # 调用开始
                data = self._choice_data(args)
                for server in self.config:
                    stype = server["type"]
                    if stype == LinkServiceType.LOCAL:
                        data = self._local(function, data)
                    else:
                        data = self._request(server, data)
                return data
            return inner
        return wrapper

    def _choice_data(self, args):
        # 自己调用的时候data可能在第1位， 服务调用的时候， 会在第2位， 第1位为self
        for idx, arg in enumerate(args):
            if idx < 2:
                if isinstance(arg, (dict, str, list, bool, int)):
                    return arg
            else:
                break
        return None

    def _local(self, function, data):
        try:
            return function(data) if data is not None else function()
        except Exception as e:
            raise LinkServerError(
                f"link server error: error server: local,  error msg: {e}")

    def _request(self, server, data):
        try:
            stype = server["type"]
            method = server["method"]
            router = server["router"]
            if stype == LinkServiceType.RPC:
                data = self._rpc_request(server["name"], router, method, data)
            elif stype == LinkServiceType.HTTP:
                data = self._http_request(router, method, data)
            if "extrainfo" in data and "data" in data:
                data = data["data"]
        except Exception as e:
            raise LinkServerError(
                f"link server error:\n server info: {server}, error msg: {e}")
        return data

    def _http_request(self, router, method, data):
        header = {"Content-Type": "application/json"}
        resp = iuap_request.sign_and_request(
            router, method, json=data, headers=header)
        return resp.json

    def _rpc_request(self, server, router, method, data):
        resp = self._client.do_service(
            server, router, return_type="json", method=method, data=data, timeout=30, prefer_ip=True)
        return resp


linkserver = LinkService() if has_eurekapy_package else LinkBase()



# regist dataset
# dont delete
from intelliw.datasets.datasource_empty import EmptyDataSource
from intelliw.datasets.datasource_intelliv import DataSourceIntelliv
from intelliw.datasets.datasource_iwfactorydata import DataSourceIwFactoryData
from intelliw.datasets.datasource_iwimgdata import DataSourceIwImgData
from intelliw.datasets.datasource_local_csv import DataSourceLocalCsv
from intelliw.datasets.datasource_nlp_corpora import DataSourceNLPCorpora
from intelliw.datasets.datasource_remote_csv import DataSourceRemoteCsv
from intelliw.datasets.datasource_semantic import DataSourceSemanticData
from intelliw.datasets.datasource_large_model import DataSourceLMCorpora
from intelliw.datasets.datasource_iwfactory_dr_data import DataSourceIwFactoryDrData



'''
Author: hexu
Date: 2021-10-14 14:54:05
LastEditTime: 2025-08-29 15:30:00
LastEditors: AI Assistant
Description: 数据工厂物化表数据源获取数据
FilePath: /iw-algo-fx/intelliw/datasets/datasource_iwfactory_dr_data.py
'''
from concurrent.futures import ThreadPoolExecutor
import os
import pandas as pd
import json
from intelliw.datasets.datasource_base import AbstractDataSource, DataSourceReaderException, DataSourceType, \
    DatasetSelector
from intelliw.utils.logger import _get_framework_logger
from intelliw.config import config
from intelliw.datasets.utils.iwfactory_dr_api import IwFactoryDrAPI

logger = _get_framework_logger()


def err_handler(request, exception):
    print("请求出错,{}".format(exception))


class DataSourceIwFactoryDrData(AbstractDataSource):
    """
    数据工场物化表数据源
    """

    def __init__(self, data_address, data_meta_address, datasoure_id, table_id, table_name, user_id, fields=None, data_filter_condition = None, tenant_id=None):
        """
        语义模型数据源
        :param data_address:   获取数据 url
        :param fields: 获取数据表字段
        :param table_id:   表Id
        """
        self.data_address = data_address
        self.data_meta_address = data_meta_address
        self.table_id = table_id
        self.user_id = user_id
        self.fields = fields
        self.datasoure_id = datasoure_id
        self.table_name = table_name
        self.__total = None
        self.__meta = []
        self.tenant_id = tenant_id if tenant_id is not None else config.TENANT_ID
        
        user_url_map = {
            'select_data': self.data_address,  # 批量查询数据
            'table_metadata': self.data_meta_address,  # 查询元数据
        }

        self.iwfactory_dr_api = IwFactoryDrAPI(
            tenant_id=self.tenant_id,
            user_id=self.user_id,
            user_url_map=user_url_map
            )

        self.conditions = None
        self.where_condition = []
        if data_filter_condition is None or len(data_filter_condition) == 0:
            data_filter_condition = config.DATA_FILTER_CONDITION
            try:
                self.conditions = json.loads(data_filter_condition).get('conditions')
                self.where_condition = self._build_conditions()
                logger.info("parse data_filter_condition success, config is %s, where_condition is %s", data_filter_condition, self.where_condition)
            except Exception as e:
                self.conditions = None
                self.where_condition = []

    def total(self):
        """获取数据总行数"""
        if self.__total is not None:
            return self.__total

        try:
            # 使用新的API接口获取数据总数和元数据
            
            # 构建查询条件
            where_condition = self.where_condition
            
            # 使用分页查询，获取第一页的数据来获取总数
            result = self.iwfactory_dr_api.select_data_paginated(
                datasource_id=self.datasoure_id,
                table_name=self.table_name,
                column_schema=[],  # 空数组表示获取所有字段
                page_size=1,
                current_page=1,
                where_condition=where_condition
            )
            
            # 获取数据模型
            data_model = result.get('data', {}).get('dataModel', {})
            page_info = data_model.get('page', {})
            
            # 获取总数
            count = page_info.get('totalCount', 0)
            if isinstance(count, int):
                self.__total = count
            else:
                self.__total = 0
            
        except Exception as e:
            msg = f"获取行数失败: {str(e)}, datasource_id: {self.datasoure_id}, table_name: {self.table_name}"
            raise DataSourceReaderException(msg)
            
        return self.__total
    
    def get_meta(self):
        if self.__meta is not None and len(self.__meta) > 0:
            return self.__meta
        try:
            # 获取字段元数据
            result = self.iwfactory_dr_api.get_table_metadata(self.table_id)
            
            # 获取字段元数据
            column_schema = result.get('data', {}).get('columns', [])
            
            # 构建字段元数据信息
            tmp_meta = []
            for col in column_schema:
                tmp_meta.append({
                    "code": col.get('columnEnglishName', ''),
                    "label": col.get('columnDescribe', ''),
                    "type": col.get('columnType', 'varchar')  # 默认类型，实际类型需要根据数据推断
                })
            
            self.__meta = tmp_meta
            return self.__meta
        except Exception as e:
            msg = f"获取元数据失败: {str(e)}, datasource_id: {self.datasoure_id}, table_name: {self.table_name}"
            raise DataSourceReaderException(msg)
    
    def _build_conditions(self):
        """将旧的过滤条件格式转换为新的API格式"""
        if not self.conditions:
            return []
        
        new_conditions = []
        for condition in self.conditions:
            # 转换旧的condition格式到新的格式
            operator_mapping = {
                'between': 'between',
                '=': '=',
                '>': '>',
                '<': '<',
                '>=': '>=',
                '<=': '<=',
                '!=': '!=',
                '<>': '!=',
                'like': 'like',
                'not like': 'not like',
                'is null': 'is null',
                'is not null': 'is not null',
                'in': 'in'
            }
            
            op = condition.get('op', '=')
            name = condition.get('name', '')
            
            new_condition = {
                "columnName": name,
                "operator": operator_mapping.get(op, op)
            }
            
            # 处理不同操作符的值
            if op == 'between':
                # between操作符需要两个值
                new_condition["columnValues"] = [condition.get('v1'), condition.get('v2')]
            elif op == 'in':
                # in操作符支持多个值
                values = condition.get('v1', '')
                if isinstance(values, str):
                    values = [v.strip() for v in values.split(',')]
                new_condition["columnValues"] = values
            elif op in ['is null', 'is not null']:
                # 空值检查不需要值
                new_condition.pop("columnValues", None)
            else:
                # 其他操作符使用单个值
                new_condition["columnValues"] = [condition.get('v1')]
            
            new_conditions.append(new_condition)
        
        return new_conditions
    def reader(self, pagesize=10000, offset=0, limit=0, transform_function=None, dataset_type='train_set'):
        return self.__Reader(self.data_address, self.datasoure_id, self.table_id, self.table_name, self.fields,self.iwfactory_dr_api, self.total(), self.get_meta(), self.user_id, pagesize, transform_function, self.where_condition)

    class __Reader:
        def __init__(self, data_address, datasoure_id, table_id, table_name, fields, iwfactory_dr_api: IwFactoryDrAPI, total, meta, user_id, pagesize=5000, transform_function=None, where_condition: list = None):
            """
            eg. 91 elements, page_size = 20, 5 pages as below:
            [0,19][20,39][40,59][60,79][80,90]
            offset 15, limit 30:
            [15,19][20,39][40,44]
            offset 10 limit 5:
            [10,14]
            """
            self.logger = _get_framework_logger()
            self.data_address = data_address
            self.datasoure_id = datasoure_id
            self.table_id = table_id
            self.table_name = table_name
            self.fields = fields
            self.iwfactory_dr_api = iwfactory_dr_api
            self.origin_page_size = pagesize
            self.page_size = max(100, pagesize)
            self.page_size = pagesize
            self.page_num = 1
            self.total_page_num = 1
            self.total_read = 0
            self.total_rows = total
            self.user_id = user_id
            self.meta = meta
            self.worker = min(3, max(1, os.cpu_count()))
            self.transform_function = transform_function
            self.where_condition = where_condition
            logger.info(f'数据获取中: 启用的获取线程 {self.worker} 条')

        def get_data_bar(self):
            """数据拉取进度条"""
            try:
                proportion = round(
                    (self.total_read / self.total_rows) * 100, 2)
                logger.info(
                    f"数据获取中: {self.total_read}/{self.total_rows}, 进度{proportion}%")
            except:
                pass

        @property
        def iterable(self):
            return True

        def __iter__(self):
            return self

        def reset(self, fields: list=None, page_size: int=None):
            """ 迭代重置 """
            self.page_num = 1
            self.total_page_num = 1
            self.total_read = 0
            self.fields = fields
            if page_size:
                self.page_size = page_size
            else:
                self.page_size = self.origin_page_size
            logger.info(f'数据读取重置, page_num: {self.page_num}, total_read: {self.total_read}, fields: {self.fields}, page_size: {self.page_size}') 

        def get_meta(self):
            """获取元数据"""
            return self.meta
        
        def get_total_rows(self):
            """获取总数据行数"""
            return self.total_rows

        

        def __next__(self):
            if self.total_read >= self.total_rows:
                logger.info('数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                raise StopIteration

            self.get_data_bar()
            self.total_page_num = (self.total_rows - 1) // self.page_size + 1
            try:
                with ThreadPoolExecutor(max_workers=self.worker) as executor:
                    futures, data_result = [], []
                    for i in range(self.worker):
                        if self.page_num > self.total_page_num:
                            break
                        futures.append(executor.submit(
                            self._read_page, self.page_num, self.page_size, self.datasoure_id, self.table_name, self.where_condition
                        ))
                        self.page_num += 1

                    for f in futures:
                        data_result.extend(f.result())
                    self.total_read += len(data_result)
                    if len(data_result) == 0:
                        logger.info(
                            '数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                        raise StopIteration

                    return_data = {"result": data_result, "meta": self.meta}
                    if self.transform_function is not None:
                        return_data = self.transform_function(return_data)
                    return return_data
            except Exception as e:
                logger.exception(
                    f"数据工厂模型数据源读取失败, data_address: [{self.table_id}]")
                raise DataSourceReaderException(f'数据工厂模型数据源读取失败: {e}')

        def _get_meta(self, meta):
            if self.meta == []:
                self.meta = meta
            return self.meta

        def _data_process(self, result):
            assert result is not None, "数据工厂模型数据接口未返回任何有效数据"
            data = result["data"]
            assert data is not None, f"接口返回无数据, 请联系数据负责人：{result}"
            if len(data) > 0:
                self._get_meta(data['meta'])
            return data['result']

        def _data_process_new(self, result):
            """处理新的物化表API响应格式"""
            assert result is not None, "物化表数据接口未返回任何有效数据"
            assert result.get("code") == 0 or result.get("code") == 200, f"接口返回错误, code: {result.get('code')}, msg: {result.get('msg')}"
            
            data = result.get("data")
            assert data is not None, f"接口返回无数据, 请联系数据负责人：{result}"
            
            data_model = data.get("dataModel")
            assert data_model is not None, f"接口返回格式错误, 无dataModel字段: {data}"
            
            # 获取列schema和模型数据
            # column_schema = data_model.get("columnSchema", [])
            model_data = data_model.get("modelData", [])
            
            return model_data

        def _read_page(self, page_index, page_size, datasoure_id, table_name, where_condition: list = []):
            """
            使用新的API接口获取分页数据
            :param page_index: 页码，从 1 开始
            :param page_size:  每页大小
            :return:
            """
            try:
                # 构建字段列表
                column_schema = self.fields if self.fields is not None else None
                
                # 使用分页查询
                result = self.iwfactory_dr_api.select_data_paginated(
                    datasource_id=datasoure_id,
                    table_name= table_name,
                    column_schema=column_schema,
                    page_size=page_size,
                    current_page=page_index,
                    where_condition=where_condition
                )
                logger.info(f"result: {result}")
                return self._data_process_new(result)
                
            except Exception as e:
                logger.error(f"分页获取数据失败: {str(e)}")
                raise DataSourceReaderException(f'分页获取数据失败: {e}')


DatasetSelector.register_func(DataSourceType.IW_FACTORY_DR_DATA, DataSourceIwFactoryDrData, {
    "data_address": "INPUT_ADDR",
    "data_meta_address": "INPUT_DATA_META_ADDR",
    "datasoure_id": "INPUT_MODEL_ID",
    "table_id": "TABLE_ID",
    "table_name": "TABLE_NAME",
    "user_id": "USER_ID",
    })



'''
Author: Hexu
Date: 2022-08-24 16:52:17
LastEditors: Hexu
LastEditTime: 2023-03-27 14:07:11
FilePath: /iw-algo-fx/intelliw/interface/trainjob.py
Description: Train entrypoint
'''
import traceback
from intelliw.core.custom import Custom
from intelliw.core.recorder import Recorder
import intelliw.utils.message as message
from intelliw.utils.logger import _get_framework_logger

logger = _get_framework_logger()


class CustomService:
    def __init__(self, path, procedure_type=None, response_addr=None):
        self.response_addr = response_addr
        self.path = path
        self.reporter = Recorder(response_addr)
        self.procedure_type = procedure_type

    def run(self):
        try:
            custom = Custom(self.path, self.response_addr, self.procedure_type)
            custom.run()
        except Exception as e:
            stack_info = traceback.format_exc()
            logger.error("自定义服务执行错误 {}, stack:\n{}".format(e, stack_info))
            self.reporter.report(
                message.CommonResponse(500, "custom_finish",
                                       "自定义服务执行错误 {}, stack:\n{}".format(e, stack_info),
                                       businessType=self.procedure_type))



'''
Author: Hexu
Date: 2022-09-26 13:34:19
LastEditors: Hexu
LastEditTime: 2022-12-26 16:18:04
FilePath: /iw-algo-fx/intelliw/utils/redis/__init__.py
Description: 
'''
import asyncio
import os

try:
    import redis
except ImportError:
    raise ImportError("\033[31mIf use redis, you need: pip install redis (>=5.0.1) \033[0m")

from .connect import RedisConnect
from .connect_async import AsyncRedisConnect
from intelliw.utils.logger import _get_framework_logger

logger = _get_framework_logger()

redis_product_code = 'iuap-aip-console-redis'


def _get_args(client_type=None, url=None, password=None, sentinel_password=None, mater_name=None, db=None,
              pool_size=None, host=None, port=None):
    rc = redis_product_code
    return (
        client_type or os.environ[f'{rc}_REDIS_TYPE'],
        url or os.environ[f'{rc}_REDIS_URL'],
        password or os.environ[f'{rc}_REDIS_PASSWORD'],
        sentinel_password or os.environ[f'{rc}_REDIS_SENTINEL_PASSWORD'],
        mater_name or os.environ[f'{rc}_REDIS_MASTER_NAME'],
        db or os.environ[f'{rc}_REDIS_DATABASE'],
        pool_size or os.environ[f'{rc}_REDIS_POOL_SIZE'],
        host or os.environ.get(f'{rc}_REDIS_HOST'),
        port or os.environ.get(f'{rc}_REDIS_PORT')
    )


def get_client(client_type=None, url=None, password=None, sentinel_password=None, master_name=None, db=None,
               pool_size=None, host=None, port=None):
    redis_client = None

    try:
        args = _get_args(client_type, url, password, sentinel_password, master_name, db, pool_size, host, port)
    except KeyError:
        logger.error('Redis args get error: yms config download error or redis config not in yms config')
    else:
        redis_client = RedisConnect(*args).redis_client

    return redis_client


def get_async_client(client_type=None, url=None, password=None, sentinel_password=None, mater_name=None, db=None,
                     pool_size=None, host=None, port=None):
    redis_client = None

    try:
        args = _get_args(client_type, url, password, sentinel_password, mater_name, db, pool_size, host, port)
        conn = AsyncRedisConnect(*args)
        # asyncio.run(conn.ping())
    except KeyError:
        logger.error('Redis args get error: yms config download error or redis config not in yms config')
    except Exception as e:
        logger.error(f'Redis connect error: {e}')
    else:
        redis_client = conn.redis_client
    return redis_client



'''
Author: hexu
Date: 2021-10-14 14:54:05
LastEditTime: 2023-03-27 11:20:17
LastEditors: Hexu
Description: 从数据工场获取模型数据集
FilePath: /iw-algo-fx/intelliw/datasets/datasource_iwfactorydata.py
'''
from concurrent.futures import ThreadPoolExecutor
import os
import pandas as pd
from intelliw.datasets.datasource_base import AbstractDataSource, DataSourceReaderException, DataSourceType, \
    DatasetSelector
from intelliw.utils import iuap_request, util
from intelliw.utils.logger import _get_framework_logger
from intelliw.config import config
from pandas.api.types import is_numeric_dtype

logger = _get_framework_logger()


def err_handler(request, exception):
    print("请求出错,{}".format(exception))


class DataSourceIwFactoryData(AbstractDataSource):
    """
    数据工场数据源
    """

    def __init__(self, input_address, get_row_address, get_table_meta, table_id, tenant_id=None):
        """
        智能分析数据源
        :param input_address:   获取数据 url
        :param get_row_address: 获取数据总条数 url
        :param table_id:   表Id
        """
        self.input_address = input_address
        self.get_row_address = get_row_address
        self.get_table_meta = get_table_meta
        self.table_id = table_id
        self.__total = None
        self.tenant_id = tenant_id if tenant_id is not None else config.TENANT_ID

    def total(self):
        """获取数据总行数"""
        if self.__total is not None:
            return self.__total
        response = iuap_request.post_json(
            self.get_row_address, json={'tableid': self.table_id, 'ytenantId': config.TENANT_ID},
            timeout=config.DATA_SOURCE_READ_TIMEOUT,
            auth_type=iuap_request.AuthType.YHT)
        if response.status != 200:
            msg = f"获取行数失败，url: {self.get_row_address}, response: {response}"
            raise DataSourceReaderException(msg)
        row_data = response.json

        try:
            count = row_data["data"]["count"]
            self.__total = count if isinstance(count, int) else 0
            return self.__total
        except Exception:
            msg = f"获取行数返回结果错误, response: {row_data}, request_url: {self.get_row_address}"
            raise DataSourceReaderException(msg)

    def table_meta(self):
        response = iuap_request.post_json(
            self.get_table_meta, json={'logicIds': [self.table_id], 'ytenantId': config.TENANT_ID},
            timeout=config.DATA_SOURCE_READ_TIMEOUT,
            auth_type=iuap_request.AuthType.YHT)
        if response.status != 200:
            msg = f"获取表信息失败，url: {self.get_table_meta}, response: {response}"
            raise DataSourceReaderException(msg)
        row_data = response.json

        try:
            meta = row_data["data"][0]["fieldList"]
            return [{'type': m['columnType'], 'code': m['columnName']} for m in meta]
        except Exception:
            msg = f"获取表信息失败, response: {row_data}, request_url: {self.get_table_meta}"
            raise DataSourceReaderException(msg)

    def reader(self, page_size=10000, offset=0, limit=0, transform_function=None, dataset_type='train_set'):
        return self.__Reader(self.input_address, self.table_id, self.table_meta(), self.total(), self.tenant_id
                             , page_size, transform_function)

    class __Reader:
        def __init__(self, input_address, table_id, table_meta, total, tenant_id, page_size=10000, transform_function=None):
            """
            eg. 91 elements, page_size = 20, 5 pages as below:
            [0,19][20,39][40,59][60,79][80,90]
            offset 15, limit 30:
            [15,19][20,39][40,44]
            offset 10 limit 5:
            [10,14]
            """
            self.input_address = input_address
            self.table_id = table_id
            self.meta = table_meta
            self.num_table_meta = self.get_number_column(self.meta)
            self.total_rows = total
            self.page_size = max(100, page_size)
            self.page_num = 1
            self.total_read = 0
            self.transform_function = transform_function
            self.worker = min(3, max(1, os.cpu_count()))
            self.tenant_id = tenant_id
            logger.info(f'数据获取中: 启用的获取线程 {self.worker} 条')

        def get_number_column(self, table_meta):
            # 所有数值型字段返回的都是字符型， 需要转一下
            number_column = []
            for m in table_meta:
                _type = m['type'].upper()
                if _type in util.DB_NUM_TYPE:
                    number_column.append(m['code'])
            return number_column

        def get_data_bar(self):
            """数据拉取进度条"""
            try:
                proportion = round(
                    (self.total_read / self.total_rows) * 100, 2)
                logger.info(
                    f"数据获取中: {self.total_read}/{self.total_rows}, 进度{proportion}%")
            except:
                pass

        @property
        def iterable(self):
            return True

        def __iter__(self):
            return self

        def __next__(self):
            if self.total_read >= self.total_rows:
                logger.info('数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                raise StopIteration

            self.get_data_bar()

            try:
                with ThreadPoolExecutor(max_workers=self.worker) as executor:
                    futures, data_result = [], []
                    for i in range(self.worker * 2):
                        futures.append(executor.submit(
                            self._read_page, self.page_num, self.page_size
                        ))
                        self.page_num += 1

                    for f in futures:
                        data_result.extend(f.result())
                    self.total_read += len(data_result)
                    if len(data_result) == 0:
                        logger.info(
                            '数据下载完成，共读取原始数据 {} 条'.format(self.total_read))
                        raise StopIteration

                    return_data = {"result": data_result, "meta": self.meta}
                    if self.transform_function is not None:
                        return_data = self.transform_function(return_data)
                    return return_data
            except StopIteration:
                raise StopIteration
            except Exception as e:
                logger.exception(
                    f"智能工场数据源读取失败, input_address: [{self.input_address}, {self.table_id}]")
                raise DataSourceReaderException(f'智能工场数据源读取失败,{e}')

        def _data_process(self, result):
            assert result is not None, "智能工场数据接口未返回任何有效数据"

            data = result['data']
            if len(data) == 0:
                return []

            # 数据过滤
            df = pd.DataFrame(data=data)
            
            # 确保返回的字段与元数据匹配
            meta_columns = [m['code'] for m in self.meta]
            # 只保留元数据中存在的字段
            df = df[[col for col in meta_columns if col in df.columns]]
            
            # 补充缺失的字段
            for col in meta_columns:
                if col not in df.columns:
                    df[col] = None
            
            # 确保列顺序与元数据一致
            df = df[meta_columns]
            
            for nm in self.num_table_meta:
                if nm in df.columns and not is_numeric_dtype(df[nm]):
                    df[nm] = pd.to_numeric(df[nm], errors='coerce')
            # 数据format
            return df.values.tolist()

        def _read_page(self, page_index, page_size):
            """
            数据工场获取数据接口，分页读取数据
            :param page_index: 页码，从 0 开始
            :param page_size:  每页大小

            此接口：pageIndex 代表起始下标（不是页）， pagesize代表每页数据的数量， pagecount代表获取几页
                   但是返回的数据类型是[{},{}] 而不是 [[{},{}],[]], 所以保证pageSize和pageCount中某一个数为1的时候， 另一个参数就可以当size使用（很迷惑）
            例如： {'id': self.table_id, 'pageIndex': 1,'pageSize': 10, 'pageCount': 1} 和 {'id': self.table_id, 'pageIndex': 1,'pageSize': 1, 'pageCount': 10} 的结果完全一致
            :return:
            """
            request_data = {
                'id': self.table_id,
                'pageIndex': page_index,
                'pageCount': page_size,
                'ytenantId': config.TENANT_ID
            }
            response = iuap_request.post_json(
                url=self.input_address, json=request_data, timeout=config.DATA_SOURCE_READ_TIMEOUT, auth_type=iuap_request.AuthType.YHT)
            response.raise_for_status()
            result = response.json
            return self._data_process(result)
        def get_meta(self):
            """获取元数据"""
            return self.meta
        
        def get_total_rows(self):
            """获取总数据行数"""
            return self.total_rows


DatasetSelector.register_func(DataSourceType.IW_FACTORY_DATA, DataSourceIwFactoryData, {
    "input_address": "INPUT_ADDR",
    "get_row_address": "INPUT_GETROW_ADDR",
    "get_table_meta": "INPUT_DATA_META_ADDR",
    "table_id": "INPUT_DATA_SOURCE_ID",
    "tenant_id": "TENANT_ID"})



#!/usr/bin/env python
# coding: utf-8
'''
Author: hexu
Date: 2021-10-25 15:20:34
LastEditTime: 2023-05-16 16:53:49
LastEditors: Hexu
Description: 线上调用的入口文件
FilePath: /iw-algo-fx/intelliw/interface/controller.py
'''
import sys
import traceback

from absl.flags import argparse_flags as argparse

from intelliw.config import config
from intelliw.utils.logger import _get_framework_logger

logger = _get_framework_logger()


class FrameworkArgs:
    """
    FrameworkArgs
    """

    def __init__(self, args=None):
        self.path = "" if args is None else args.path
        self.method = "importalg" if args is None else args.method
        self.name = "predict" if args is None else args.name
        self.format = "" if args is None else args.format
        self.task = "infer" if args is None else args.task
        self.port = 8888 if args is None else args.port
        self.response = None if args is None else args.response
        self.output = "" if args is None else args.output


def __parse_args():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-p", "--path", default="",
                        type=str, help="package path")
    parser.add_argument("-m", "--method", default="importalg",
                        type=str, help="method")
    parser.add_argument("-n", "--name", default="predict",
                        type=str, help="name")
    parser.add_argument("-o", "--output", default="", type=str, help="output")
    parser.add_argument("-f", "--format", default="",
                        type=str, help="batch format")
    parser.add_argument("-t", "--task", default="infer",
                        type=str, help="task type: infer/train")
    parser.add_argument("--port", default=8888, type=int, help="port")
    parser.add_argument("--instance_id", default="", type=str, help="instance id")
    parser.add_argument("-r", "--response",
                        default=None, type=str,
                        help="response addr, which can be used to report status")

    return parser.parse_args()


def main(args):
    """
    intelliw main
    """
    try:
        config.FRAMEWORK_MODE = {
            "importalg": config.FrameworkMode.Import,
            "importmodel": config.FrameworkMode.Import,
            "train": config.FRAMEWORK_MODE or config.FrameworkMode.Train,
            "apiservice": config.FrameworkMode.Infer,
            "batchservice": config.FrameworkMode.Batch,
            "custom": config.FRAMEWORK_MODE
        }.get(args.method, config.FRAMEWORK_MODE)

        if args.method == "importalg":
            from intelliw.core.pipeline import Pipeline
            Pipeline(args.response).importalg(args.path, False)
        elif args.method == "importmodel":
            from intelliw.core.pipeline import Pipeline
            Pipeline(args.response).importmodel(args.path, False)
        elif args.method == "train":
            from intelliw.interface.trainjob import TrainServer
            train = TrainServer(args.path, config.DATASET_INFO, args.response)
            train.run()
        elif args.method == "apiservice":
            from intelliw.interface.iwapi.iwapp import ApiService
            api_service = ApiService(args.port, args.path, args.response)
            api_service.run()
        elif args.method == "batchservice":
            from intelliw.interface.batchjob import BatchService
            batch_service = BatchService(
                args.format, args.path, config.DATASET_INFO,
                config.OUTPUT_DATASET_INFO, args.response, args.task)
            batch_service.run()
        elif args.method == "custom":
            from intelliw.interface.customjob import CustomService
            custom_service = CustomService(args.path, config.FRAMEWORK_MODE, args.response)
            custom_service.run()

        sys.exit(0)
    except Exception:
        logger.error("fail to execute and stack:\n%s", traceback.format_exc())
        sys.exit(1)


def run():
    """
    run intelliw
    """
    framework_args = FrameworkArgs(__parse_args())
    # 初始化环境变量
    config.update_by_env()
    main(framework_args)


if __name__ == '__main__':
    run()



'''
Author: hexu
Date: 2021-10-14 14:54:05
LastEditTime: 2023-05-23 18:47:07
LastEditors: Hexu
Description: 远程csv数据集
FilePath: /iw-algo-fx/intelliw/datasets/datasource_remote_csv.py
'''
import time

from intelliw.datasets.datasource_base import AbstractDataSource, DatasetSelector, DataSourceType
from intelliw.datasets.datasource_local_csv import DataSourceLocalCsv
from intelliw.utils.exception import DatasetException
from intelliw.utils.logger import _get_framework_logger
import traceback
from intelliw.utils.storage_service import StorageService

logger = _get_framework_logger()


class DataSourceRemoteCsv(AbstractDataSource):
    """
    远程 csv 文件数据源
    """

    def __init__(self, source_address, download_path='./tmp_csv_file_{}.csv'):
        self.source_address = source_address
        self.__tmp_csv_file_path = download_path.format(int(time.time() * 1e6))
        self.__download_file()
        self.__local_csv_data_source = DataSourceLocalCsv(
            self.__tmp_csv_file_path)

    def total(self):
        return self.__local_csv_data_source.total()

    def get_meta(self):
        return self.__local_csv_data_source.get_meta()

    def reader(self, pagesize=10000, offset=0, limit=0, transform_function=None, dataset_type='train_set'):
        return self.__local_csv_data_source.reader(pagesize, offset, limit, transform_function)

    def __download_file(self):
        start_time = time.time()
        logger.info('Downloading csv files from %s to %s',
                    self.source_address, self.__tmp_csv_file_path)
        download_file_name = self.source_address
        local_file_path = self.__tmp_csv_file_path

        try:
            downloader = StorageService(
                download_file_name, "download")
            downloader.download(local_file_path, stream=True)
            logger.info(f"Csv下载成功, 耗时:{time.time() - start_time}s")
        except Exception as e:
            err = traceback.format_exc()
            raise DatasetException(f"Csv下载失败: {err}")


DatasetSelector.register_func(DataSourceType.REMOTE_CSV, DataSourceRemoteCsv, {
    "source_address": "DATA_SOURCE_ADDRESS"})



from intelliw.config.cfg_parser import load_config
import yaml
import os
from functools import wraps


def verify_and_generate_model_cfg(func):
    @wraps(func)
    def verify(*args):
        if not os.path.exists(args[1]):
            if not os.path.exists(args[0]):
                raise ValueError(
                    "missing algorithm.yaml at path: {}".format(args[0]))
            print('model.yaml does not exists, generating model.yaml to [{}] based on [{}].'.format(
                args[1], args[0]))
            func(*args)

    return verify


@verify_and_generate_model_cfg
def generate_model_config_from_algorithm_config(algorithm_config_path, model_config_path, location="model"):
    """
    根据 algorithm.yaml 生成 model.yaml

    :param algorithm_config_path: algorithm.yaml path
    :param model_config_path: model.yaml path
    :param location: model path
    """
    algo_cfg = load_config(algorithm_config_path)
    model_cfg = {
        "Model": {
            "name": algo_cfg["AlgorithmInformation"]["name"],
            "desc": algo_cfg["AlgorithmInformation"]["desc"],
            "location": location,
            "algorithm": {
                "name": algo_cfg["AlgorithmInformation"]["name"],
                "desc": algo_cfg["AlgorithmInformation"]["desc"],
            }
        }
    }
    _algo_cfg = algo_cfg["AlgorithmInformation"]["algorithm"]
    if "metadata" in _algo_cfg:
        model_cfg["Model"]["metadata"] = _algo_cfg["metadata"]
    if "transforms" in _algo_cfg:
        model_cfg["Model"]["transforms"] = _algo_cfg["transforms"]
    if "parameters" in _algo_cfg and isinstance(_algo_cfg["parameters"], list):
        params = {}
        for item in _algo_cfg["parameters"]:
            if "key" in item and "val" in item:
                params[item["key"]] = item["val"]
        model_cfg["Model"]["parameters"] = params
    if "command" in algo_cfg["AlgorithmInformation"]:
        command = algo_cfg["AlgorithmInformation"].get("command", [])
        args = algo_cfg["AlgorithmInformation"].get("args", [])
        if not args and command:
            raise Exception("自定义启动命令command不能为空")
        for a in args:
            command.extend([a['key'], a['val']])
        model_cfg["Model"]["boot"] = command
    if "system" in algo_cfg["AlgorithmInformation"]:
        params = {}
        for item in algo_cfg["AlgorithmInformation"]['system']["parameters"]:
            if "key" in item and "val" in item:
                params[item["key"]] = item["val"]
        model_cfg["Model"]["system"] = params
    model = yaml.dump(model_cfg, default_flow_style=False, allow_unicode=True)

    f = open(model_config_path, 'w', encoding='utf-8')
    f.write(model)
    f.close()
